
// Yacc parser template (TP Yacc V3.0), V1.2 6-17-91 AG *)
//
// Modified to be used with Delphi 7 by Jan Punter.



// /////////////////////////////////////////////////////////////////////////////
//
//  Nord Modular patch file format 3.03 parser
//  Copyright (C) 2003 Marcus Andersson
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  ////////////////////////////////////////////////////////////////////////////
//
//  Modified to be used in Delphi 7 by Jan Punter.
//
//  Modifications CopyRight (c) 2003 Jan Punter
//
//  Distributed under the same conditions as the original code by Narcus
//  Andersson.
//
//  This should be compileable for the tp yacc vession 3.0 as taken from
//  the free pascal distribution. Modified versions of lexlib.pas, yacclib.pas,
//  yyylex.cod and yyparse.cod were used. These should be present in this
//  distribution.
//
//  Notes :
//
//    Take care with quoting in the comments above (and below), an uneven number
//    of quotes in this section will give compile errors from pyacc.
//
//    The lexical analyzer created from pdllex.l is included in the code
//    generated from this file, and it _must_ be included.
//
//    lupd : 2003-10-16
//
//  ////////////////////////////////////////////////////////////////////////////



Unit patchparser;

Interface

{$WARN UNSAFE_TYPE OFF}

Uses

  SysUtils, Classes, patchstructs;


Type

  YYSType = String;

Var

  GPatch          : TPatch;
  GSection        : TModuleSection;
  GModule         : TModule;
  GMorph          : TMorph;
  GMorphMap       : TMorphMap;
  GKnobMap        : TKnobMap;
  GCtrlMap        : TCtrlMap;
  GParameterIndex : Integer;
  GCustomIndex    : Integer;
  GStrings        : TStringList;


  Procedure InitPatch( aPatch: TPatch);

const PARAM = 257;
const NMVERSION = 258;
const HEADER = 259;
const HEADER_END = 260;
const MODULE_DUMP = 261;
const MODULE_DUMP_END = 262;
const CURRENT_NOTE_DUMP = 263;
const CURRENT_NOTE_DUMP_END = 264;
const CABLE_DUMP = 265;
const CABLE_DUMP_END = 266;
const PARAMETER_DUMP = 267;
const PARAMETER_DUMP_END = 268;
const CUSTOM_DUMP = 269;
const CUSTOM_DUMP_END = 270;
const MORPH_MAP_DUMP = 271;
const MORPH_MAP_DUMP_END = 272;
const KEYBOARD_ASSIGNMENT = 273;
const KEYBOARD_ASSIGNMENT_END = 274;
const KNOB_MAP_DUMP = 275;
const KNOB_MAP_DUMP_END = 276;
const CTRL_MAP_DUMP = 277;
const CTRL_MAP_DUMP_END = 278;
const NAME_DUMP = 279;
const NAME_DUMP_END = 280;
const NOTES = 281;

Type

  // Exceptions

  EParser    = Class( Exception);
  ESingleton = Class( EParser  );


  // Callbacks

  TParseError = Procedure( aSender: TObject; Const aMsg: String; Var AbortParsing: Boolean) Of Object;


  // TParser, a wrapper around the Lex / Yacc generated parser.

	TParser = Class
  Private
    FDebug        : Boolean;
    FOnParseError : TParseError;
  Private
    Function    GetToken: Integer;
    Procedure   SetDebug( aValue: Boolean);
    Function    GetErrorCount: Integer;
    Function    GetMessages  : TStrings;
    Function    GetErrorFile : String;
    Function    GetErrorLine : Integer;
    Function    GetErrorCol  : Integer;
	Public
    Constructor Create;
    Destructor  Destroy;                                               Override;
		Function    Parse: Integer;
    Procedure   Clear;
    Procedure   PushFile( Const aFileName: String);
    Procedure   ParseError( Const aMsg: String);
    Procedure   ParseErrorFmt( Const aFormat: String; Const anArgs: Array Of Const);
  Public
    Property    Debug        : Boolean     Read FDebug        Write SetDebug;
    Property    ErrorCount   : Integer     Read GetErrorCount;
    Property    OnParseError : TParseError Read FOnParseError Write FOnParseError;
    Property    Messages     : TStrings    Read GetMessages;
    Property    ErrorFile    : String      Read GetErrorFile;
    Property    ErrorLine    : Integer     Read GetErrorLine;
    Property    ErrorCol     : Integer     Read GetErrorCol;
	End;


Var

  yylval : YYSType;


Implementation

Uses

  lexlib, yacclib;

Var

  ConstructCount : Integer = 0;


{ ========
	TParser = Class
  Private
    FDebug        : Boolean;
    FOnParseError : TParseError;
  Public
    Property    Debug        : Boolean     Read FDebug        Write SetDebug;
    Property    ErrorCount   : Integer     Read GetErrorCount;
    Property    OnParseError : TParseError Read FOnParseError Write FOnParseError;
    Property    ErrorFile    : String      Read GetErrorFile;
    Property    ErrorLine    : Integer     Read GetErrorLine;
    Propert     ErrorCol     : Integer     Read GetErrorCol;
  Private
}

    Procedure   TParser.SetDebug( aValue: Boolean);
    Begin
      FDebug  := aValue;
      yyDebug := FDebug;
    End;

    Function    TParser.GetErrorCount: Integer;
    Begin
      Result := yyErrorCount;
    End;

    Function    TParser.GetMessages: TStrings;
    Begin
      Result := yyOutput;
    End;

    Function    TParser.GetErrorFile: String;
    Begin
      Result := yyFileName;
    End;

    Function    TParser.GetErrorLine: Integer;
    Begin
      Result := yyLineNo;
    End;

    Function    TParser.GetErrorCol: Integer;
    Begin
      Result := yyColNo;
    End;

//  Public

    Constructor TParser.Create;
    Begin
      If ConstructCount > 0
      Then Raise ESingleton.Create( 'Only one parser may be constructed.');
      Inc( ConstructCount);
      Inherited;
      Clear;
    End;

    Destructor  TParser.Destroy; // Override;
    Begin
      If ConstructCount > 0
      Then Dec( ConstructCount);
      Inherited;
    End;

    Procedure   TParser.Clear;
    Begin
      yyInit;
    End;

    Procedure   TParser.PushFile( Const aFileName: String);
    Begin
      If Not FileExists( aFileName)
      Then ParseErrorFmt( 'TParser.PushFile : File not fcound : ''%s''', [ aFileName]);
      yyInclude( aFileName);
    End;

    Procedure   TParser.ParseError( Const aMsg: String);
    Var
      S            : String;
      AbortParsing : Boolean;
    Begin
      S :=
        Format(
          'Error [file=''%s'' line=%d col=%d ] ''%s''',
          [
            ErrorFile,
            ErrorLine,
            ErrorCol,
            aMsg
          ]
        );
      If Assigned( FOnParseError)
      Then Begin
        AbortParsing := True;
        FOnParseError( Self, S, AbortParsing);
        If AbortParsing
        Then Abort;
      End
      Else Raise EParser.Create( S);
    End;

    Procedure   TParser.ParseErrorFmt( Const aFormat: String; Const anArgs: Array Of Const);
    Begin
      ParseError( Format( aFormat, anArgs));
    End;


Function TParser.Parse: Integer;
Var
  yystate : Integer;
  yysp    : Integer;
  yyn     : Integer;
  yys     : array [ 1 .. yymaxdepth] Of Integer;
  yyv     : array [ 1 .. yymaxdepth] Of YYSType;
  yyval   : YYSType;


  Procedure yyaction( yyruleno: Integer);
  // local definitions:


  Begin
    // actions:
    Case yyruleno Of

   1 : begin
         yyval := yyv[yysp-1];
       end;
   2 : begin
       end;
   3 : begin
         yyval := yyv[yysp-0];
       end;
   4 : begin
         yyval := yyv[yysp-0];
       end;
   5 : begin
         yyval := yyv[yysp-0];
       end;
   6 : begin
         yyval := yyv[yysp-0];
       end;
   7 : begin
         yyval := yyv[yysp-0];
       end;
   8 : begin
         yyval := yyv[yysp-0];
       end;
   9 : begin
         yyval := yyv[yysp-0];
       end;
  10 : begin
         yyval := yyv[yysp-0];
       end;
  11 : begin
         yyval := yyv[yysp-0];
       end;
  12 : begin
         yyval := yyv[yysp-0];
       end;
  13 : begin
         yyval := yyv[yysp-0];
       end;
  14 : begin
         yyval := yyv[yysp-0];
       end;
  15 : begin
         yyval := yyv[yysp-0];
       end;
  16 : begin
         yyval := yyv[yysp-0];
       end;
  17 : begin
         yyval := yyv[yysp-0];
       end;
  18 : begin
         
         // header : HEADER '\n' NMVERSION '\n' PARAM (13,13) '\n' HEADER_END _
         
         With GPatch Do
         Begin
         KeyboardRange[ rlMin] := StrToInt( Trim(  yyv[yysp-24]));
         KeyboardRange[ rlMax] := StrToInt( Trim(  yyv[yysp-23]));
         VelocityRange[ rlMin] := StrToInt( Trim(  yyv[yysp-22]));
         VelocityRange[ rlMax] := StrToInt( Trim(  yyv[yysp-21]));
         BendRange             := StrToInt( Trim(  yyv[yysp-20]));
         PortamentoTime        := StrToInt( Trim( yyv[yysp-19]));
         Portamento            := TPortamento( StrToInt( Trim( yyv[yysp-18])));
         RequestedVoices       := StrToInt( Trim( yyv[yysp-17]));
         SectionSeparationPos  := StrToInt( Trim( yyv[yysp-16]));
         OctaveShift           := StrToInt( Trim( yyv[yysp-15]));
         ModuleSection[ msPoly  ].VoiceRetrigger := TVoiceRetrigger( StrToInt( Trim( yyv[yysp-14])));
         ModuleSection[ msCommon].VoiceRetrigger := TVoiceRetrigger( StrToInt( Trim( yyv[yysp-13])));
         CableVisibility[ coRED   ] := Boolean( StrToInt( Trim( yyv[yysp-8])));
         CableVisibility[ coBLUE  ] := Boolean( StrToInt( Trim( yyv[yysp-7])));
         CableVisibility[ coYELLOW] := Boolean( StrToInt( Trim( yyv[yysp-6])));
         CableVisibility[ coGRAY  ] := Boolean( StrToInt( Trim( yyv[yysp-5])));
         CableVisibility[ coGREEN ] := Boolean( StrToInt( Trim( yyv[yysp-4])));
         CableVisibility[ coPURPLE] := Boolean( StrToInt( Trim( yyv[yysp-3])));
         CableVisibility[ coWHITE ] := Boolean( StrToInt( Trim( yyv[yysp-2])));
         End;
         
       end;
  19 : begin
         
         // module_dump : MODULE_DUMP '\n' PARAM '\n' _ module_list MODULE_DUMP_END
         
         GSection := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( yyv[yysp-1])))];
         
       end;
  20 : begin
         yyval := yyv[yysp-6];
       end;
  21 : begin
         
         // module_dump_300 : MODULE_DUMP '\n' PARAM _ module_list MODULE_DUMP_END
         
         GSection := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( yyv[yysp-0])))];
         
       end;
  22 : begin
         yyval := yyv[yysp-5];
       end;
  23 : begin
         
         // module_list : module_list PARAM (4,4) _
         
         GModule := GSection.NewModule( StrToInt( Trim( yyv[yysp-3])), StrToInt( Trim( yyv[yysp-4])));
         With GModule Do
         Begin
         XPosition := StrToInt( Trim( yyv[yysp-2]));
         YPosition := StrToInt( Trim( yyv[yysp-1]));
         End;
         
       end;
  24 : begin
         yyval := yyv[yysp-0];
       end;
  25 : begin
       end;
  26 : begin
         yyval := yyv[yysp-4];
       end;
  27 : begin
         
         // note_list : note_list PARAM (3,3) _
         
         GPatch.NewMidiNote( StrToInt( Trim( yyv[yysp-2])), StrToInt( Trim( yyv[yysp-1])), StrToInt( Trim( yyv[yysp-0])));
         
       end;
  28 : begin
       end;
  29 : begin
         
         // cable_dump : CABLE_DUMP '\n' PARAM '\n' _ cable_list CABLE_DUMP_END
         
         GSection := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( yyv[yysp-1])))];
         
       end;
  30 : begin
         yyval := yyv[yysp-6];
       end;
  31 : begin
         
         // cable_dump_300 : CABLE_DUMP '\n' PARAM _ cable_list CABLE_DUMP_END
         
         GSection := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( yyv[yysp-0])))];
         
       end;
  32 : begin
         yyval := yyv[yysp-5];
       end;
  33 : begin
         
         // cable_list : cable_list PARAM (7,7) '\n' _
         
         GSection.
         NewCable(
         TTColor(     StrToInt( Trim( yyv[yysp-7]))),
         StrToInt( Trim( yyv[yysp-6])) ,
         StrToInt( Trim( yyv[yysp-5])) ,
         TTConnector( StrToInt( Trim( yyv[yysp-4]))),
         StrToInt( Trim( yyv[yysp-3])) ,
         StrToInt( Trim( yyv[yysp-2])) ,
         TTConnector( StrToInt( Trim( yyv[yysp-1])))
         );
         
       end;
  34 : begin
         yyval := yyv[yysp-0];
       end;
  35 : begin
       end;
  36 : begin
         
         // @@@@1
         // parameter_dump : PARAMETER_DUMP '\n' PARAM '\n' _ module_parameter_list PARAMETER_DUMP_END
         
         GSection := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( yyv[yysp-1])))];
         
       end;
  37 : begin
         yyval := yyv[yysp-6];
       end;
  38 : begin
         
         // module_parameter_list : module_parameter_list PARAM (3,3) _ parameter_list '\n'
         
         GModule := GSection.Module[ StrToInt( Trim( yyv[yysp-2]))];
         GParameterIndex := 0;
         
       end;
  39 : begin
         yyval := yyv[yysp-6];
       end;
  40 : begin
         yyval := yyv[yysp-0];
       end;
  41 : begin
       end;
  42 : begin
         
         // parameter_list : parameter_list PARAM _
         
         GModule.Parameter[ GParameterIndex] := StrToInt( Trim( yyv[yysp-0]));
         Inc( GParameterIndex);
         
       end;
  43 : begin
       end;
  44 : begin
         
         // custom_dump CUSTOM_DUMP '\n' PARAM _ module_custom_list CUSTOM_DUMP_END
         
         GSection := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( yyv[yysp-0])))];
         
       end;
  45 : begin
         yyval := yyv[yysp-5];
       end;
  46 : begin
         
         // module_custom_list : module_custom_list PARAM (2,2) _ custom_list '\n'
         
         GModule := GSection.Module[ StrToInt( Trim( yyv[yysp-1]))];
         GCustomIndex := 0;
         
       end;
  47 : begin
         yyval := yyv[yysp-5];
       end;
  48 : begin
         yyval := yyv[yysp-0];
       end;
  49 : begin
       end;
  50 : begin
         
         // custom_list : custom_list PARAM _
         
         GModule.Custom[ GCustomIndex] := StrToInt( Trim( yyv[yysp-0]));
         Inc( GCustomIndex);
         
       end;
  51 : begin
       end;
  52 : begin
         
         // morph_map_dump : MORPH_MAP_DUMP '\n' PARAM (4,4) '\n' _ morph_map_list '\n' MORPH_MAP_DUMP_END
         
         With GPatch Do
         Begin
         Morph[ mtMorph1].Value := StrToInt( Trim( yyv[yysp-4]));
         Morph[ mtMorph2].Value := StrToInt( Trim( yyv[yysp-3]));
         Morph[ mtMorph3].Value := StrToInt( Trim( yyv[yysp-2]));
         Morph[ mtMorph4].Value := StrToInt( Trim( yyv[yysp-1]));
         End;
         
       end;
  53 : begin
         yyval := yyv[yysp-10];
       end;
  54 : begin
         
         // morph_map_list : morph_map_list PARAM (5,5) _
         
         GSection  := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( yyv[yysp-4])))];
         GModule   := GSection.Module[ StrToInt( Trim( yyv[yysp-3]))];
         GMorph    := GPatch.Morph[ TTMorph( StrToInt( Trim( yyv[yysp-1])))];
         GMorphMap := GMorph.NewMorphMap( TTModuleSection( StrToInt( Trim( yyv[yysp-4]))), GModule, StrToInt( Trim( yyv[yysp-2])));
         GMorphMap.Range := StrToInt( Trim( yyv[yysp-0]));
         
       end;
  55 : begin
       end;
  56 : begin
         
         // keyboard_assignment : KEYBOARD_ASSIGNMENT '\n' PARAM (4,4) '\n' _ KEYBOARD_ASSIGNMENT_END
         
         With Gpatch Do
         Begin
         Morph[ mtMorph1].KeyboardAssignment := TKeyboardAssignment( StrToInt( Trim( yyv[yysp-4])));
         Morph[ mtMorph2].KeyboardAssignment := TKeyboardAssignment( StrToInt( Trim( yyv[yysp-3])));
         Morph[ mtMorph3].KeyboardAssignment := TKeyboardAssignment( StrToInt( Trim( yyv[yysp-2])));
         Morph[ mtMorph4].KeyboardAssignment := TKeyboardAssignment( StrToInt( Trim( yyv[yysp-1])));
         End;
         
       end;
  57 : begin
         yyval := yyv[yysp-8];
       end;
  58 : begin
         yyval := yyv[yysp-3];
       end;
  59 : begin
         
         // knob_map_list : knob_list_map PARAM (4,4) '\n' _
         
         If  StrToInt( Trim( yyv[yysp-4])) = Ord( msMorph)
         Then GModule := Nil
         Else Begin
         GSection := GPatch.ModuleSection[ TTmoduleSection( StrToInt( Trim( yyv[yysp-4])))];
         GModule  := GSection.Module[ StrToInt( Trim( yyv[yysp-3]))];
         End;
         GKnobMap := GPatch.NewKnobMap( TTModuleSection( StrToInt( Trim( yyv[yysp-4]))), GModule, StrToInt( Trim( yyv[yysp-2])));
         GKnobMap.Knob := StrToInt( Trim( yyv[yysp-1]));
         
       end;
  60 : begin
         yyval := yyv[yysp-0];
       end;
  61 : begin
       end;
  62 : begin
         yyval := yyv[yysp-3];
       end;
  63 : begin
         
         // ctrl_map_list : ctrl_map_list PARAM (4,4) '\n' _
         
         If  StrToInt( Trim( yyv[yysp-4])) = Ord( msMorph)
         Then GModule := Nil
         Else Begin
         GSection := GPatch.ModuleSection[ TTmoduleSection( StrToInt( Trim( yyv[yysp-4])))];
         GModule  := GSection.Module[ StrToInt( Trim( yyv[yysp-3]))];
         End;
         GCtrlMap := GPatch.NewCtrlMap( TTModuleSection( StrToInt( Trim( yyv[yysp-4]))), GModule, StrToInt( Trim( yyv[yysp-2])));
         GCtrlMap.CC := StrToInt( Trim( yyv[yysp-1]));
         
       end;
  64 : begin
         yyval := yyv[yysp-0];
       end;
  65 : begin
       end;
  66 : begin
         
         // name_dump : NAMEDUMP '\n' PARAM '\n' _ name_list NAME_DUMP_END
         
         GSection := GPatch.ModuleSection[ TTmoduleSection( StrToInt( Trim( yyv[yysp-1])))];
         
       end;
  67 : begin
         yyval := yyv[yysp-6];
       end;
  68 : begin
         
         // name_list : name_list PARAM name '\n' _
         
         GModule := GSection.Module[ StrToInt( Trim( yyv[yysp-2]))];
         GModule.Name := yyv[yysp-1];
         
       end;
  69 : begin
       end;
  70 : begin
         
         // name : name PARAM _
         
         yyval := yyv[yysp-1] + yyv[yysp-0];
         
       end;
  71 : begin
         
         // name : _
         
         yyval := '';
         
       end;
  72 : begin
         
         // notes : NOTES _
         
         With GStrings Do
         Begin
         SetText( PChar( yyv[yysp-0]));
         If ( Count > 0) And ( Strings[ 0] = '')
         Then Delete( 0);
         End;
         GPatch.Notes := GStrings;
         
       end;

    End;
  End; // yyAction

  // ///////////////////////////////////////////////////////////////////////////

// parse table:


type YYARec = record
                sym, act : Integer;
              end;
     YYRRec = record
                len, sym : Integer;
              end;

const

yynacts   = 158;
yyngotos  = 41;
yynstates = 175;
yynrules  = 72;

yya : array [1..yynacts] of YYARec = (
{ 0: }
{ 1: }
  ( sym: 0; act: 0 ),
  ( sym: 10; act: 17 ),
  ( sym: 259; act: 18 ),
  ( sym: 261; act: 19 ),
  ( sym: 263; act: 20 ),
  ( sym: 265; act: 21 ),
  ( sym: 267; act: 22 ),
  ( sym: 269; act: 23 ),
  ( sym: 271; act: 24 ),
  ( sym: 273; act: 25 ),
  ( sym: 275; act: 26 ),
  ( sym: 277; act: 27 ),
  ( sym: 279; act: 28 ),
  ( sym: 281; act: 29 ),
{ 2: }
{ 3: }
{ 4: }
{ 5: }
{ 6: }
{ 7: }
{ 8: }
{ 9: }
{ 10: }
{ 11: }
{ 12: }
{ 13: }
{ 14: }
{ 15: }
{ 16: }
{ 17: }
{ 18: }
  ( sym: 10; act: 30 ),
{ 19: }
  ( sym: 10; act: 31 ),
{ 20: }
  ( sym: 10; act: 32 ),
{ 21: }
  ( sym: 10; act: 33 ),
{ 22: }
  ( sym: 10; act: 34 ),
{ 23: }
  ( sym: 10; act: 35 ),
{ 24: }
  ( sym: 10; act: 36 ),
{ 25: }
  ( sym: 10; act: 37 ),
{ 26: }
  ( sym: 10; act: 38 ),
{ 27: }
  ( sym: 10; act: 39 ),
{ 28: }
  ( sym: 10; act: 40 ),
{ 29: }
{ 30: }
  ( sym: 258; act: 41 ),
{ 31: }
  ( sym: 257; act: 42 ),
{ 32: }
{ 33: }
  ( sym: 257; act: 44 ),
{ 34: }
  ( sym: 257; act: 45 ),
{ 35: }
  ( sym: 257; act: 46 ),
{ 36: }
  ( sym: 257; act: 47 ),
{ 37: }
  ( sym: 257; act: 48 ),
{ 38: }
  ( sym: 10; act: 50 ),
  ( sym: 257; act: -61 ),
  ( sym: 276; act: -61 ),
{ 39: }
  ( sym: 10; act: 52 ),
  ( sym: 257; act: -65 ),
  ( sym: 278; act: -65 ),
{ 40: }
  ( sym: 257; act: 53 ),
{ 41: }
  ( sym: 10; act: 54 ),
{ 42: }
  ( sym: 10; act: 56 ),
  ( sym: 257; act: -21 ),
  ( sym: 262; act: -21 ),
{ 43: }
  ( sym: 10; act: 57 ),
  ( sym: 257; act: 58 ),
{ 44: }
  ( sym: 10; act: 60 ),
  ( sym: 257; act: -31 ),
  ( sym: 266; act: -31 ),
{ 45: }
  ( sym: 10; act: 61 ),
{ 46: }
{ 47: }
  ( sym: 257; act: 63 ),
{ 48: }
  ( sym: 257; act: 64 ),
{ 49: }
  ( sym: 257; act: 65 ),
  ( sym: 276; act: 66 ),
{ 50: }
{ 51: }
  ( sym: 257; act: 67 ),
  ( sym: 278; act: 68 ),
{ 52: }
{ 53: }
  ( sym: 10; act: 69 ),
{ 54: }
  ( sym: 257; act: 70 ),
{ 55: }
  ( sym: 10; act: 72 ),
  ( sym: 257; act: -25 ),
  ( sym: 262; act: -25 ),
{ 56: }
{ 57: }
  ( sym: 264; act: 74 ),
{ 58: }
  ( sym: 257; act: 75 ),
{ 59: }
  ( sym: 10; act: 77 ),
  ( sym: 257; act: -35 ),
  ( sym: 266; act: -35 ),
{ 60: }
{ 61: }
{ 62: }
  ( sym: 10; act: 81 ),
  ( sym: 257; act: -49 ),
  ( sym: 270; act: -49 ),
{ 63: }
  ( sym: 257; act: 82 ),
{ 64: }
  ( sym: 257; act: 83 ),
{ 65: }
  ( sym: 257; act: 84 ),
{ 66: }
{ 67: }
  ( sym: 257; act: 85 ),
{ 68: }
{ 69: }
{ 70: }
  ( sym: 257; act: 87 ),
{ 71: }
  ( sym: 257; act: 88 ),
  ( sym: 262; act: 89 ),
{ 72: }
{ 73: }
  ( sym: 10; act: 72 ),
  ( sym: 257; act: -25 ),
  ( sym: 262; act: -25 ),
{ 74: }
{ 75: }
  ( sym: 257; act: 91 ),
{ 76: }
  ( sym: 257; act: 92 ),
  ( sym: 266; act: 93 ),
{ 77: }
{ 78: }
  ( sym: 10; act: 77 ),
  ( sym: 257; act: -35 ),
  ( sym: 266; act: -35 ),
{ 79: }
  ( sym: 10; act: 96 ),
  ( sym: 257; act: -41 ),
  ( sym: 268; act: -41 ),
{ 80: }
  ( sym: 257; act: 97 ),
  ( sym: 270; act: 98 ),
{ 81: }
{ 82: }
  ( sym: 257; act: 99 ),
{ 83: }
  ( sym: 257; act: 100 ),
{ 84: }
  ( sym: 257; act: 101 ),
{ 85: }
  ( sym: 257; act: 102 ),
{ 86: }
{ 87: }
  ( sym: 257; act: 104 ),
{ 88: }
  ( sym: 257; act: 105 ),
{ 89: }
{ 90: }
  ( sym: 257; act: 88 ),
  ( sym: 262; act: 106 ),
{ 91: }
{ 92: }
  ( sym: 257; act: 107 ),
{ 93: }
{ 94: }
  ( sym: 257; act: 92 ),
  ( sym: 266; act: 108 ),
{ 95: }
  ( sym: 257; act: 109 ),
  ( sym: 268; act: 110 ),
{ 96: }
{ 97: }
  ( sym: 257; act: 111 ),
{ 98: }
{ 99: }
  ( sym: 10; act: 112 ),
{ 100: }
  ( sym: 10; act: 113 ),
{ 101: }
  ( sym: 257; act: 114 ),
{ 102: }
  ( sym: 257; act: 115 ),
{ 103: }
  ( sym: 257; act: 116 ),
  ( sym: 280; act: 117 ),
{ 104: }
  ( sym: 257; act: 118 ),
{ 105: }
  ( sym: 257; act: 119 ),
{ 106: }
{ 107: }
  ( sym: 257; act: 120 ),
{ 108: }
{ 109: }
  ( sym: 257; act: 121 ),
{ 110: }
{ 111: }
{ 112: }
{ 113: }
{ 114: }
  ( sym: 10; act: 125 ),
{ 115: }
  ( sym: 10; act: 126 ),
{ 116: }
{ 117: }
{ 118: }
  ( sym: 257; act: 128 ),
{ 119: }
  ( sym: 257; act: 129 ),
{ 120: }
  ( sym: 257; act: 130 ),
{ 121: }
  ( sym: 257; act: 131 ),
{ 122: }
{ 123: }
{ 124: }
  ( sym: 274; act: 134 ),
{ 125: }
{ 126: }
{ 127: }
  ( sym: 10; act: 135 ),
  ( sym: 257; act: 136 ),
{ 128: }
  ( sym: 257; act: 137 ),
{ 129: }
  ( sym: 10; act: 138 ),
{ 130: }
  ( sym: 257; act: 139 ),
{ 131: }
{ 132: }
  ( sym: 10; act: 141 ),
  ( sym: 257; act: 142 ),
{ 133: }
  ( sym: 10; act: 143 ),
  ( sym: 257; act: 144 ),
{ 134: }
{ 135: }
{ 136: }
{ 137: }
  ( sym: 257; act: 145 ),
{ 138: }
{ 139: }
  ( sym: 257; act: 146 ),
{ 140: }
{ 141: }
{ 142: }
{ 143: }
  ( sym: 272; act: 148 ),
{ 144: }
  ( sym: 257; act: 149 ),
{ 145: }
  ( sym: 257; act: 150 ),
{ 146: }
  ( sym: 257; act: 151 ),
{ 147: }
  ( sym: 10; act: 152 ),
  ( sym: 257; act: 153 ),
{ 148: }
{ 149: }
  ( sym: 257; act: 154 ),
{ 150: }
  ( sym: 257; act: 155 ),
{ 151: }
  ( sym: 10; act: 156 ),
{ 152: }
{ 153: }
{ 154: }
  ( sym: 257; act: 157 ),
{ 155: }
  ( sym: 257; act: 158 ),
{ 156: }
{ 157: }
  ( sym: 257; act: 159 ),
{ 158: }
  ( sym: 257; act: 160 ),
{ 159: }
{ 160: }
  ( sym: 257; act: 161 ),
{ 161: }
  ( sym: 257; act: 162 ),
{ 162: }
  ( sym: 257; act: 163 ),
{ 163: }
  ( sym: 257; act: 164 ),
{ 164: }
  ( sym: 257; act: 165 ),
{ 165: }
  ( sym: 257; act: 166 ),
{ 166: }
  ( sym: 257; act: 167 ),
{ 167: }
  ( sym: 257; act: 168 ),
{ 168: }
  ( sym: 257; act: 169 ),
{ 169: }
  ( sym: 257; act: 170 ),
{ 170: }
  ( sym: 257; act: 171 ),
{ 171: }
  ( sym: 257; act: 172 ),
{ 172: }
  ( sym: 10; act: 173 ),
{ 173: }
  ( sym: 260; act: 174 )
{ 174: }
);

yyg : array [1..yyngotos] of YYARec = (
{ 0: }
  ( sym: -2; act: 1 ),
{ 1: }
  ( sym: -17; act: 2 ),
  ( sym: -16; act: 3 ),
  ( sym: -15; act: 4 ),
  ( sym: -14; act: 5 ),
  ( sym: -13; act: 6 ),
  ( sym: -12; act: 7 ),
  ( sym: -11; act: 8 ),
  ( sym: -10; act: 9 ),
  ( sym: -9; act: 10 ),
  ( sym: -8; act: 11 ),
  ( sym: -7; act: 12 ),
  ( sym: -6; act: 13 ),
  ( sym: -5; act: 14 ),
  ( sym: -4; act: 15 ),
  ( sym: -3; act: 16 ),
{ 2: }
{ 3: }
{ 4: }
{ 5: }
{ 6: }
{ 7: }
{ 8: }
{ 9: }
{ 10: }
{ 11: }
{ 12: }
{ 13: }
{ 14: }
{ 15: }
{ 16: }
{ 17: }
{ 18: }
{ 19: }
{ 20: }
{ 21: }
{ 22: }
{ 23: }
{ 24: }
{ 25: }
{ 26: }
{ 27: }
{ 28: }
{ 29: }
{ 30: }
{ 31: }
{ 32: }
  ( sym: -21; act: 43 ),
{ 33: }
{ 34: }
{ 35: }
{ 36: }
{ 37: }
{ 38: }
  ( sym: -36; act: 49 ),
{ 39: }
  ( sym: -37; act: 51 ),
{ 40: }
{ 41: }
{ 42: }
  ( sym: -20; act: 55 ),
{ 43: }
{ 44: }
  ( sym: -24; act: 59 ),
{ 45: }
{ 46: }
  ( sym: -30; act: 62 ),
{ 47: }
{ 48: }
{ 49: }
{ 50: }
{ 51: }
{ 52: }
{ 53: }
{ 54: }
{ 55: }
  ( sym: -18; act: 71 ),
{ 56: }
  ( sym: -19; act: 73 ),
{ 57: }
{ 58: }
{ 59: }
  ( sym: -22; act: 76 ),
{ 60: }
  ( sym: -23; act: 78 ),
{ 61: }
  ( sym: -26; act: 79 ),
{ 62: }
  ( sym: -29; act: 80 ),
{ 63: }
{ 64: }
{ 65: }
{ 66: }
{ 67: }
{ 68: }
{ 69: }
  ( sym: -39; act: 86 ),
{ 70: }
{ 71: }
{ 72: }
{ 73: }
  ( sym: -18; act: 90 ),
{ 74: }
{ 75: }
{ 76: }
{ 77: }
{ 78: }
  ( sym: -22; act: 94 ),
{ 79: }
  ( sym: -25; act: 95 ),
{ 80: }
{ 81: }
{ 82: }
{ 83: }
{ 84: }
{ 85: }
{ 86: }
  ( sym: -38; act: 103 ),
{ 87: }
{ 88: }
{ 89: }
{ 90: }
{ 91: }
{ 92: }
{ 93: }
{ 94: }
{ 95: }
{ 96: }
{ 97: }
{ 98: }
{ 99: }
{ 100: }
{ 101: }
{ 102: }
{ 103: }
{ 104: }
{ 105: }
{ 106: }
{ 107: }
{ 108: }
{ 109: }
{ 110: }
{ 111: }
  ( sym: -32; act: 122 ),
{ 112: }
  ( sym: -34; act: 123 ),
{ 113: }
  ( sym: -35; act: 124 ),
{ 114: }
{ 115: }
{ 116: }
  ( sym: -40; act: 127 ),
{ 117: }
{ 118: }
{ 119: }
{ 120: }
{ 121: }
{ 122: }
  ( sym: -31; act: 132 ),
{ 123: }
  ( sym: -33; act: 133 ),
{ 124: }
{ 125: }
{ 126: }
{ 127: }
{ 128: }
{ 129: }
{ 130: }
{ 131: }
  ( sym: -28; act: 140 ),
{ 132: }
{ 133: }
{ 134: }
{ 135: }
{ 136: }
{ 137: }
{ 138: }
{ 139: }
{ 140: }
  ( sym: -27; act: 147 )
{ 141: }
{ 142: }
{ 143: }
{ 144: }
{ 145: }
{ 146: }
{ 147: }
{ 148: }
{ 149: }
{ 150: }
{ 151: }
{ 152: }
{ 153: }
{ 154: }
{ 155: }
{ 156: }
{ 157: }
{ 158: }
{ 159: }
{ 160: }
{ 161: }
{ 162: }
{ 163: }
{ 164: }
{ 165: }
{ 166: }
{ 167: }
{ 168: }
{ 169: }
{ 170: }
{ 171: }
{ 172: }
{ 173: }
{ 174: }
);

yyd : array [0..yynstates-1] of Integer = (
{ 0: } -2,
{ 1: } 0,
{ 2: } -16,
{ 3: } -15,
{ 4: } -14,
{ 5: } -13,
{ 6: } -12,
{ 7: } -11,
{ 8: } -10,
{ 9: } -9,
{ 10: } -8,
{ 11: } -7,
{ 12: } -6,
{ 13: } -5,
{ 14: } -4,
{ 15: } -3,
{ 16: } -1,
{ 17: } -17,
{ 18: } 0,
{ 19: } 0,
{ 20: } 0,
{ 21: } 0,
{ 22: } 0,
{ 23: } 0,
{ 24: } 0,
{ 25: } 0,
{ 26: } 0,
{ 27: } 0,
{ 28: } 0,
{ 29: } -72,
{ 30: } 0,
{ 31: } 0,
{ 32: } -28,
{ 33: } 0,
{ 34: } 0,
{ 35: } 0,
{ 36: } 0,
{ 37: } 0,
{ 38: } 0,
{ 39: } 0,
{ 40: } 0,
{ 41: } 0,
{ 42: } 0,
{ 43: } 0,
{ 44: } 0,
{ 45: } 0,
{ 46: } -44,
{ 47: } 0,
{ 48: } 0,
{ 49: } 0,
{ 50: } -60,
{ 51: } 0,
{ 52: } -64,
{ 53: } 0,
{ 54: } 0,
{ 55: } 0,
{ 56: } -19,
{ 57: } 0,
{ 58: } 0,
{ 59: } 0,
{ 60: } -29,
{ 61: } -36,
{ 62: } 0,
{ 63: } 0,
{ 64: } 0,
{ 65: } 0,
{ 66: } -58,
{ 67: } 0,
{ 68: } -62,
{ 69: } -66,
{ 70: } 0,
{ 71: } 0,
{ 72: } -24,
{ 73: } 0,
{ 74: } -26,
{ 75: } 0,
{ 76: } 0,
{ 77: } -34,
{ 78: } 0,
{ 79: } 0,
{ 80: } 0,
{ 81: } -48,
{ 82: } 0,
{ 83: } 0,
{ 84: } 0,
{ 85: } 0,
{ 86: } -69,
{ 87: } 0,
{ 88: } 0,
{ 89: } -22,
{ 90: } 0,
{ 91: } -27,
{ 92: } 0,
{ 93: } -32,
{ 94: } 0,
{ 95: } 0,
{ 96: } -40,
{ 97: } 0,
{ 98: } -45,
{ 99: } 0,
{ 100: } 0,
{ 101: } 0,
{ 102: } 0,
{ 103: } 0,
{ 104: } 0,
{ 105: } 0,
{ 106: } -20,
{ 107: } 0,
{ 108: } -30,
{ 109: } 0,
{ 110: } -37,
{ 111: } -46,
{ 112: } -52,
{ 113: } -56,
{ 114: } 0,
{ 115: } 0,
{ 116: } -71,
{ 117: } -67,
{ 118: } 0,
{ 119: } 0,
{ 120: } 0,
{ 121: } 0,
{ 122: } -51,
{ 123: } -55,
{ 124: } 0,
{ 125: } -59,
{ 126: } -63,
{ 127: } 0,
{ 128: } 0,
{ 129: } 0,
{ 130: } 0,
{ 131: } -38,
{ 132: } 0,
{ 133: } 0,
{ 134: } -57,
{ 135: } -68,
{ 136: } -70,
{ 137: } 0,
{ 138: } -23,
{ 139: } 0,
{ 140: } -43,
{ 141: } -47,
{ 142: } -50,
{ 143: } 0,
{ 144: } 0,
{ 145: } 0,
{ 146: } 0,
{ 147: } 0,
{ 148: } -53,
{ 149: } 0,
{ 150: } 0,
{ 151: } 0,
{ 152: } -39,
{ 153: } -42,
{ 154: } 0,
{ 155: } 0,
{ 156: } -33,
{ 157: } 0,
{ 158: } 0,
{ 159: } -54,
{ 160: } 0,
{ 161: } 0,
{ 162: } 0,
{ 163: } 0,
{ 164: } 0,
{ 165: } 0,
{ 166: } 0,
{ 167: } 0,
{ 168: } 0,
{ 169: } 0,
{ 170: } 0,
{ 171: } 0,
{ 172: } 0,
{ 173: } 0,
{ 174: } -18
);

yyal : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 1,
{ 2: } 15,
{ 3: } 15,
{ 4: } 15,
{ 5: } 15,
{ 6: } 15,
{ 7: } 15,
{ 8: } 15,
{ 9: } 15,
{ 10: } 15,
{ 11: } 15,
{ 12: } 15,
{ 13: } 15,
{ 14: } 15,
{ 15: } 15,
{ 16: } 15,
{ 17: } 15,
{ 18: } 15,
{ 19: } 16,
{ 20: } 17,
{ 21: } 18,
{ 22: } 19,
{ 23: } 20,
{ 24: } 21,
{ 25: } 22,
{ 26: } 23,
{ 27: } 24,
{ 28: } 25,
{ 29: } 26,
{ 30: } 26,
{ 31: } 27,
{ 32: } 28,
{ 33: } 28,
{ 34: } 29,
{ 35: } 30,
{ 36: } 31,
{ 37: } 32,
{ 38: } 33,
{ 39: } 36,
{ 40: } 39,
{ 41: } 40,
{ 42: } 41,
{ 43: } 44,
{ 44: } 46,
{ 45: } 49,
{ 46: } 50,
{ 47: } 50,
{ 48: } 51,
{ 49: } 52,
{ 50: } 54,
{ 51: } 54,
{ 52: } 56,
{ 53: } 56,
{ 54: } 57,
{ 55: } 58,
{ 56: } 61,
{ 57: } 61,
{ 58: } 62,
{ 59: } 63,
{ 60: } 66,
{ 61: } 66,
{ 62: } 66,
{ 63: } 69,
{ 64: } 70,
{ 65: } 71,
{ 66: } 72,
{ 67: } 72,
{ 68: } 73,
{ 69: } 73,
{ 70: } 73,
{ 71: } 74,
{ 72: } 76,
{ 73: } 76,
{ 74: } 79,
{ 75: } 79,
{ 76: } 80,
{ 77: } 82,
{ 78: } 82,
{ 79: } 85,
{ 80: } 88,
{ 81: } 90,
{ 82: } 90,
{ 83: } 91,
{ 84: } 92,
{ 85: } 93,
{ 86: } 94,
{ 87: } 94,
{ 88: } 95,
{ 89: } 96,
{ 90: } 96,
{ 91: } 98,
{ 92: } 98,
{ 93: } 99,
{ 94: } 99,
{ 95: } 101,
{ 96: } 103,
{ 97: } 103,
{ 98: } 104,
{ 99: } 104,
{ 100: } 105,
{ 101: } 106,
{ 102: } 107,
{ 103: } 108,
{ 104: } 110,
{ 105: } 111,
{ 106: } 112,
{ 107: } 112,
{ 108: } 113,
{ 109: } 113,
{ 110: } 114,
{ 111: } 114,
{ 112: } 114,
{ 113: } 114,
{ 114: } 114,
{ 115: } 115,
{ 116: } 116,
{ 117: } 116,
{ 118: } 116,
{ 119: } 117,
{ 120: } 118,
{ 121: } 119,
{ 122: } 120,
{ 123: } 120,
{ 124: } 120,
{ 125: } 121,
{ 126: } 121,
{ 127: } 121,
{ 128: } 123,
{ 129: } 124,
{ 130: } 125,
{ 131: } 126,
{ 132: } 126,
{ 133: } 128,
{ 134: } 130,
{ 135: } 130,
{ 136: } 130,
{ 137: } 130,
{ 138: } 131,
{ 139: } 131,
{ 140: } 132,
{ 141: } 132,
{ 142: } 132,
{ 143: } 132,
{ 144: } 133,
{ 145: } 134,
{ 146: } 135,
{ 147: } 136,
{ 148: } 138,
{ 149: } 138,
{ 150: } 139,
{ 151: } 140,
{ 152: } 141,
{ 153: } 141,
{ 154: } 141,
{ 155: } 142,
{ 156: } 143,
{ 157: } 143,
{ 158: } 144,
{ 159: } 145,
{ 160: } 145,
{ 161: } 146,
{ 162: } 147,
{ 163: } 148,
{ 164: } 149,
{ 165: } 150,
{ 166: } 151,
{ 167: } 152,
{ 168: } 153,
{ 169: } 154,
{ 170: } 155,
{ 171: } 156,
{ 172: } 157,
{ 173: } 158,
{ 174: } 159
);

yyah : array [0..yynstates-1] of Integer = (
{ 0: } 0,
{ 1: } 14,
{ 2: } 14,
{ 3: } 14,
{ 4: } 14,
{ 5: } 14,
{ 6: } 14,
{ 7: } 14,
{ 8: } 14,
{ 9: } 14,
{ 10: } 14,
{ 11: } 14,
{ 12: } 14,
{ 13: } 14,
{ 14: } 14,
{ 15: } 14,
{ 16: } 14,
{ 17: } 14,
{ 18: } 15,
{ 19: } 16,
{ 20: } 17,
{ 21: } 18,
{ 22: } 19,
{ 23: } 20,
{ 24: } 21,
{ 25: } 22,
{ 26: } 23,
{ 27: } 24,
{ 28: } 25,
{ 29: } 25,
{ 30: } 26,
{ 31: } 27,
{ 32: } 27,
{ 33: } 28,
{ 34: } 29,
{ 35: } 30,
{ 36: } 31,
{ 37: } 32,
{ 38: } 35,
{ 39: } 38,
{ 40: } 39,
{ 41: } 40,
{ 42: } 43,
{ 43: } 45,
{ 44: } 48,
{ 45: } 49,
{ 46: } 49,
{ 47: } 50,
{ 48: } 51,
{ 49: } 53,
{ 50: } 53,
{ 51: } 55,
{ 52: } 55,
{ 53: } 56,
{ 54: } 57,
{ 55: } 60,
{ 56: } 60,
{ 57: } 61,
{ 58: } 62,
{ 59: } 65,
{ 60: } 65,
{ 61: } 65,
{ 62: } 68,
{ 63: } 69,
{ 64: } 70,
{ 65: } 71,
{ 66: } 71,
{ 67: } 72,
{ 68: } 72,
{ 69: } 72,
{ 70: } 73,
{ 71: } 75,
{ 72: } 75,
{ 73: } 78,
{ 74: } 78,
{ 75: } 79,
{ 76: } 81,
{ 77: } 81,
{ 78: } 84,
{ 79: } 87,
{ 80: } 89,
{ 81: } 89,
{ 82: } 90,
{ 83: } 91,
{ 84: } 92,
{ 85: } 93,
{ 86: } 93,
{ 87: } 94,
{ 88: } 95,
{ 89: } 95,
{ 90: } 97,
{ 91: } 97,
{ 92: } 98,
{ 93: } 98,
{ 94: } 100,
{ 95: } 102,
{ 96: } 102,
{ 97: } 103,
{ 98: } 103,
{ 99: } 104,
{ 100: } 105,
{ 101: } 106,
{ 102: } 107,
{ 103: } 109,
{ 104: } 110,
{ 105: } 111,
{ 106: } 111,
{ 107: } 112,
{ 108: } 112,
{ 109: } 113,
{ 110: } 113,
{ 111: } 113,
{ 112: } 113,
{ 113: } 113,
{ 114: } 114,
{ 115: } 115,
{ 116: } 115,
{ 117: } 115,
{ 118: } 116,
{ 119: } 117,
{ 120: } 118,
{ 121: } 119,
{ 122: } 119,
{ 123: } 119,
{ 124: } 120,
{ 125: } 120,
{ 126: } 120,
{ 127: } 122,
{ 128: } 123,
{ 129: } 124,
{ 130: } 125,
{ 131: } 125,
{ 132: } 127,
{ 133: } 129,
{ 134: } 129,
{ 135: } 129,
{ 136: } 129,
{ 137: } 130,
{ 138: } 130,
{ 139: } 131,
{ 140: } 131,
{ 141: } 131,
{ 142: } 131,
{ 143: } 132,
{ 144: } 133,
{ 145: } 134,
{ 146: } 135,
{ 147: } 137,
{ 148: } 137,
{ 149: } 138,
{ 150: } 139,
{ 151: } 140,
{ 152: } 140,
{ 153: } 140,
{ 154: } 141,
{ 155: } 142,
{ 156: } 142,
{ 157: } 143,
{ 158: } 144,
{ 159: } 144,
{ 160: } 145,
{ 161: } 146,
{ 162: } 147,
{ 163: } 148,
{ 164: } 149,
{ 165: } 150,
{ 166: } 151,
{ 167: } 152,
{ 168: } 153,
{ 169: } 154,
{ 170: } 155,
{ 171: } 156,
{ 172: } 157,
{ 173: } 158,
{ 174: } 158
);

yygl : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 2,
{ 2: } 17,
{ 3: } 17,
{ 4: } 17,
{ 5: } 17,
{ 6: } 17,
{ 7: } 17,
{ 8: } 17,
{ 9: } 17,
{ 10: } 17,
{ 11: } 17,
{ 12: } 17,
{ 13: } 17,
{ 14: } 17,
{ 15: } 17,
{ 16: } 17,
{ 17: } 17,
{ 18: } 17,
{ 19: } 17,
{ 20: } 17,
{ 21: } 17,
{ 22: } 17,
{ 23: } 17,
{ 24: } 17,
{ 25: } 17,
{ 26: } 17,
{ 27: } 17,
{ 28: } 17,
{ 29: } 17,
{ 30: } 17,
{ 31: } 17,
{ 32: } 17,
{ 33: } 18,
{ 34: } 18,
{ 35: } 18,
{ 36: } 18,
{ 37: } 18,
{ 38: } 18,
{ 39: } 19,
{ 40: } 20,
{ 41: } 20,
{ 42: } 20,
{ 43: } 21,
{ 44: } 21,
{ 45: } 22,
{ 46: } 22,
{ 47: } 23,
{ 48: } 23,
{ 49: } 23,
{ 50: } 23,
{ 51: } 23,
{ 52: } 23,
{ 53: } 23,
{ 54: } 23,
{ 55: } 23,
{ 56: } 24,
{ 57: } 25,
{ 58: } 25,
{ 59: } 25,
{ 60: } 26,
{ 61: } 27,
{ 62: } 28,
{ 63: } 29,
{ 64: } 29,
{ 65: } 29,
{ 66: } 29,
{ 67: } 29,
{ 68: } 29,
{ 69: } 29,
{ 70: } 30,
{ 71: } 30,
{ 72: } 30,
{ 73: } 30,
{ 74: } 31,
{ 75: } 31,
{ 76: } 31,
{ 77: } 31,
{ 78: } 31,
{ 79: } 32,
{ 80: } 33,
{ 81: } 33,
{ 82: } 33,
{ 83: } 33,
{ 84: } 33,
{ 85: } 33,
{ 86: } 33,
{ 87: } 34,
{ 88: } 34,
{ 89: } 34,
{ 90: } 34,
{ 91: } 34,
{ 92: } 34,
{ 93: } 34,
{ 94: } 34,
{ 95: } 34,
{ 96: } 34,
{ 97: } 34,
{ 98: } 34,
{ 99: } 34,
{ 100: } 34,
{ 101: } 34,
{ 102: } 34,
{ 103: } 34,
{ 104: } 34,
{ 105: } 34,
{ 106: } 34,
{ 107: } 34,
{ 108: } 34,
{ 109: } 34,
{ 110: } 34,
{ 111: } 34,
{ 112: } 35,
{ 113: } 36,
{ 114: } 37,
{ 115: } 37,
{ 116: } 37,
{ 117: } 38,
{ 118: } 38,
{ 119: } 38,
{ 120: } 38,
{ 121: } 38,
{ 122: } 38,
{ 123: } 39,
{ 124: } 40,
{ 125: } 40,
{ 126: } 40,
{ 127: } 40,
{ 128: } 40,
{ 129: } 40,
{ 130: } 40,
{ 131: } 40,
{ 132: } 41,
{ 133: } 41,
{ 134: } 41,
{ 135: } 41,
{ 136: } 41,
{ 137: } 41,
{ 138: } 41,
{ 139: } 41,
{ 140: } 41,
{ 141: } 42,
{ 142: } 42,
{ 143: } 42,
{ 144: } 42,
{ 145: } 42,
{ 146: } 42,
{ 147: } 42,
{ 148: } 42,
{ 149: } 42,
{ 150: } 42,
{ 151: } 42,
{ 152: } 42,
{ 153: } 42,
{ 154: } 42,
{ 155: } 42,
{ 156: } 42,
{ 157: } 42,
{ 158: } 42,
{ 159: } 42,
{ 160: } 42,
{ 161: } 42,
{ 162: } 42,
{ 163: } 42,
{ 164: } 42,
{ 165: } 42,
{ 166: } 42,
{ 167: } 42,
{ 168: } 42,
{ 169: } 42,
{ 170: } 42,
{ 171: } 42,
{ 172: } 42,
{ 173: } 42,
{ 174: } 42
);

yygh : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 16,
{ 2: } 16,
{ 3: } 16,
{ 4: } 16,
{ 5: } 16,
{ 6: } 16,
{ 7: } 16,
{ 8: } 16,
{ 9: } 16,
{ 10: } 16,
{ 11: } 16,
{ 12: } 16,
{ 13: } 16,
{ 14: } 16,
{ 15: } 16,
{ 16: } 16,
{ 17: } 16,
{ 18: } 16,
{ 19: } 16,
{ 20: } 16,
{ 21: } 16,
{ 22: } 16,
{ 23: } 16,
{ 24: } 16,
{ 25: } 16,
{ 26: } 16,
{ 27: } 16,
{ 28: } 16,
{ 29: } 16,
{ 30: } 16,
{ 31: } 16,
{ 32: } 17,
{ 33: } 17,
{ 34: } 17,
{ 35: } 17,
{ 36: } 17,
{ 37: } 17,
{ 38: } 18,
{ 39: } 19,
{ 40: } 19,
{ 41: } 19,
{ 42: } 20,
{ 43: } 20,
{ 44: } 21,
{ 45: } 21,
{ 46: } 22,
{ 47: } 22,
{ 48: } 22,
{ 49: } 22,
{ 50: } 22,
{ 51: } 22,
{ 52: } 22,
{ 53: } 22,
{ 54: } 22,
{ 55: } 23,
{ 56: } 24,
{ 57: } 24,
{ 58: } 24,
{ 59: } 25,
{ 60: } 26,
{ 61: } 27,
{ 62: } 28,
{ 63: } 28,
{ 64: } 28,
{ 65: } 28,
{ 66: } 28,
{ 67: } 28,
{ 68: } 28,
{ 69: } 29,
{ 70: } 29,
{ 71: } 29,
{ 72: } 29,
{ 73: } 30,
{ 74: } 30,
{ 75: } 30,
{ 76: } 30,
{ 77: } 30,
{ 78: } 31,
{ 79: } 32,
{ 80: } 32,
{ 81: } 32,
{ 82: } 32,
{ 83: } 32,
{ 84: } 32,
{ 85: } 32,
{ 86: } 33,
{ 87: } 33,
{ 88: } 33,
{ 89: } 33,
{ 90: } 33,
{ 91: } 33,
{ 92: } 33,
{ 93: } 33,
{ 94: } 33,
{ 95: } 33,
{ 96: } 33,
{ 97: } 33,
{ 98: } 33,
{ 99: } 33,
{ 100: } 33,
{ 101: } 33,
{ 102: } 33,
{ 103: } 33,
{ 104: } 33,
{ 105: } 33,
{ 106: } 33,
{ 107: } 33,
{ 108: } 33,
{ 109: } 33,
{ 110: } 33,
{ 111: } 34,
{ 112: } 35,
{ 113: } 36,
{ 114: } 36,
{ 115: } 36,
{ 116: } 37,
{ 117: } 37,
{ 118: } 37,
{ 119: } 37,
{ 120: } 37,
{ 121: } 37,
{ 122: } 38,
{ 123: } 39,
{ 124: } 39,
{ 125: } 39,
{ 126: } 39,
{ 127: } 39,
{ 128: } 39,
{ 129: } 39,
{ 130: } 39,
{ 131: } 40,
{ 132: } 40,
{ 133: } 40,
{ 134: } 40,
{ 135: } 40,
{ 136: } 40,
{ 137: } 40,
{ 138: } 40,
{ 139: } 40,
{ 140: } 41,
{ 141: } 41,
{ 142: } 41,
{ 143: } 41,
{ 144: } 41,
{ 145: } 41,
{ 146: } 41,
{ 147: } 41,
{ 148: } 41,
{ 149: } 41,
{ 150: } 41,
{ 151: } 41,
{ 152: } 41,
{ 153: } 41,
{ 154: } 41,
{ 155: } 41,
{ 156: } 41,
{ 157: } 41,
{ 158: } 41,
{ 159: } 41,
{ 160: } 41,
{ 161: } 41,
{ 162: } 41,
{ 163: } 41,
{ 164: } 41,
{ 165: } 41,
{ 166: } 41,
{ 167: } 41,
{ 168: } 41,
{ 169: } 41,
{ 170: } 41,
{ 171: } 41,
{ 172: } 41,
{ 173: } 41,
{ 174: } 41
);

yyr : array [1..yynrules] of YYRRec = (
{ 1: } ( len: 2; sym: -2 ),
{ 2: } ( len: 0; sym: -2 ),
{ 3: } ( len: 1; sym: -3 ),
{ 4: } ( len: 1; sym: -3 ),
{ 5: } ( len: 1; sym: -3 ),
{ 6: } ( len: 1; sym: -3 ),
{ 7: } ( len: 1; sym: -3 ),
{ 8: } ( len: 1; sym: -3 ),
{ 9: } ( len: 1; sym: -3 ),
{ 10: } ( len: 1; sym: -3 ),
{ 11: } ( len: 1; sym: -3 ),
{ 12: } ( len: 1; sym: -3 ),
{ 13: } ( len: 1; sym: -3 ),
{ 14: } ( len: 1; sym: -3 ),
{ 15: } ( len: 1; sym: -3 ),
{ 16: } ( len: 1; sym: -3 ),
{ 17: } ( len: 1; sym: -3 ),
{ 18: } ( len: 29; sym: -4 ),
{ 19: } ( len: 0; sym: -19 ),
{ 20: } ( len: 7; sym: -5 ),
{ 21: } ( len: 0; sym: -20 ),
{ 22: } ( len: 6; sym: -6 ),
{ 23: } ( len: 6; sym: -18 ),
{ 24: } ( len: 1; sym: -18 ),
{ 25: } ( len: 0; sym: -18 ),
{ 26: } ( len: 5; sym: -7 ),
{ 27: } ( len: 4; sym: -21 ),
{ 28: } ( len: 0; sym: -21 ),
{ 29: } ( len: 0; sym: -23 ),
{ 30: } ( len: 7; sym: -8 ),
{ 31: } ( len: 0; sym: -24 ),
{ 32: } ( len: 6; sym: -9 ),
{ 33: } ( len: 9; sym: -22 ),
{ 34: } ( len: 1; sym: -22 ),
{ 35: } ( len: 0; sym: -22 ),
{ 36: } ( len: 0; sym: -26 ),
{ 37: } ( len: 7; sym: -10 ),
{ 38: } ( len: 0; sym: -28 ),
{ 39: } ( len: 7; sym: -25 ),
{ 40: } ( len: 1; sym: -25 ),
{ 41: } ( len: 0; sym: -25 ),
{ 42: } ( len: 2; sym: -27 ),
{ 43: } ( len: 0; sym: -27 ),
{ 44: } ( len: 0; sym: -30 ),
{ 45: } ( len: 6; sym: -11 ),
{ 46: } ( len: 0; sym: -32 ),
{ 47: } ( len: 6; sym: -29 ),
{ 48: } ( len: 1; sym: -29 ),
{ 49: } ( len: 0; sym: -29 ),
{ 50: } ( len: 2; sym: -31 ),
{ 51: } ( len: 0; sym: -31 ),
{ 52: } ( len: 0; sym: -34 ),
{ 53: } ( len: 11; sym: -12 ),
{ 54: } ( len: 6; sym: -33 ),
{ 55: } ( len: 0; sym: -33 ),
{ 56: } ( len: 0; sym: -35 ),
{ 57: } ( len: 9; sym: -13 ),
{ 58: } ( len: 4; sym: -14 ),
{ 59: } ( len: 6; sym: -36 ),
{ 60: } ( len: 1; sym: -36 ),
{ 61: } ( len: 0; sym: -36 ),
{ 62: } ( len: 4; sym: -15 ),
{ 63: } ( len: 6; sym: -37 ),
{ 64: } ( len: 1; sym: -37 ),
{ 65: } ( len: 0; sym: -37 ),
{ 66: } ( len: 0; sym: -39 ),
{ 67: } ( len: 7; sym: -16 ),
{ 68: } ( len: 4; sym: -38 ),
{ 69: } ( len: 0; sym: -38 ),
{ 70: } ( len: 2; sym: -40 ),
{ 71: } ( len: 0; sym: -40 ),
{ 72: } ( len: 1; sym: -17 )
);


Const

  _error = 256; // error token

  Function yyact( state, sym: Integer; Var act: Integer): Boolean;
  // search action table
  Var
    k : Integer;
  Begin
    k := yyal[ state];
    while ( k <= yyah[ state]) And ( yya[ k].sym <> sym) Do
      Inc( k);
    If k > yyah[ state]
    Then Result := false
    Else Begin
      act    := yya[ k].act;
      Result := true;
    End;
  End;


  Function yygoto( state, sym: Integer; Var nstate: Integer): Boolean;
  // search goto table
  Var
    k : Integer;
  Begin
    k := yygl[ state];
    While ( k <= yygh[ state]) And ( yyg[ k].sym <> sym) Do
      Inc( k);
    If k > yygh[ state]
    Then Result := false
    Else Begin
      nstate := yyg[k].act;
      Result := true;
    End;
  End;


  Label

    parse,
    next,
    error,
    errlab,
    shift,
    reduce,
    accept,
    abort;

  Begin // yyparse

    // initialize:

    yystate := 0; yychar := -1; yynerrs := 0; yyerrflag := 0; yysp := 0;

  parse:

    // push state and value:

    inc( yysp);
    If yysp > yymaxdepth
    Then
    Begin
      ParseError( 'yyparse stack overflow');
      Goto abort;
    end;
    yys[ yysp] := yystate;
    yyv[ yysp] := yyval;

  next:

    If ( yyd[ yystate] = 0) And ( yychar = -1)
    Then // get next symbol
    Begin
      yychar := GetToken;
      If yychar < 0
      Then yychar := 0;
    End;

    yyDebugOutFmt(
      'state %d, char %d, ''%s''',
      [
        yystate,
        yychar,
        Printable( yychar)
      ]
    );

    // determine parse action:

    yyn := yyd[ yystate];
    If yyn <> 0
    Then Goto reduce; // simple state

    // no default action; search parse table

    If Not yyact( yystate, yychar, yyn)
    Then Goto error
    Else If yyn > 0
    Then Goto shift
    Else If yyn < 0
    Then Goto reduce
    Else Goto accept;

  error:

  // error; start error recovery:

    If yyerrflag = 0
    Then ParseError( 'syntax error');

  errlab:

    If yyerrflag = 0
    Then inc(yynerrs);     // new error

    If yyerrflag<=2
    Then Begin             // incomplete recovery; try again
      yyerrflag := 3;

      // uncover a state with shift action on error token
      While
        ( yysp > 0) And Not ( yyact( yys[ yysp], _error, yyn) And ( yyn > 0))
      Do Begin
        If yydebug
        Then
          If yysp > 1
          Then
            yyDebugOutFmt(
              'error recovery pops state %d, uncovers %d',
              [
                yys[ yysp],
                yys[ yysp - 1]
              ]
            )
          Else
            yyDebugOut( 'error ecovery fails ... abort');
        Dec( yysp);
      End;
      If yysp = 0
      Then Goto abort;   // parser has fallen from stack; abort
      yystate := yyn;    // simulate shift on error
      Goto parse;
    End
    Else Begin // no shift yet; discard symbol
      yyDebugOutFmt(
        'error recovery discards char %d (%s)',
        [
          yychar,
          Printable( yychar)
        ]
      );
      If yychar = 0
      Then Goto abort;           // end of input; abort
      yychar := -1;
      Goto next;                 // clear lookahead char and try again
    End;

  shift:

    // go to new state, clear lookahead character:

    yystate := yyn;
    yychar  := -1;
    yyval   := yylval;
    If yyerrflag > 0
    Then Dec( yyerrflag);

    Goto parse;

  reduce:

    // execute action, pop rule from stack, and go to next state:

    yyDebugOutFmt( 'reduce %d', [ -yyn]);

    yyflag := yyfnone;
    yyaction( -yyn);
    dec( yysp, yyr[ -yyn].len);
    If yygoto( yys[ yysp], yyr[ -yyn].sym, yyn)
    Then yystate := yyn;

    // handle action calls to yyaccept, yyabort and yyerror:

    Case yyflag Of
      yyfaccept : goto accept;
      yyfabort  : goto abort;
      yyferror  : goto errlab;
    End;

    Goto parse;

  accept:

    Result := 0;
    exit;

  abort:

    Result := 1;
    exit;

End; // yyparse



  Procedure InitPatch( aPatch: TPatch);
  Begin
    GPatch := aPatch;
  End;

  
// /////////////////////////////////////////////////////////////////////////////
// Include the lexer

{$INCLUDE patchlexer.pas}

// End  of lexer.
// /////////////////////////////////////////////////////////////////////////////


Initialization

  GStrings := TStringList.Create;

Finalization

  If Assigned( GStrings)
  Then FreeAndNil( GStrings);

End.

