Unit patchstructs;

//  ////////////////////////////////////////////////////////////////////////////
//
//  Nord Modular patch file format 3.03 parser
//  Copyright (C) 2003 Marcus Andersson
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  ////////////////////////////////////////////////////////////////////////////
//
//  Translated into Pascal by Jan Punter, meant to be compiled under Delphi 7.
//
//  Modifications CopyRight (c) 2003 Jan Punter
//
//  Distributed under the same conditions as the original code by Narcus
//  Andersson.
//
//    2003-10-16
//
//  Had to change the loading constructor for Patch from the original code into
//  a split Create and LoadPatch method for TPatch. Changed some names and
//  changed some types.
//
//  ////////////////////////////////////////////////////////////////////////////

Interface

Uses

  SysUtils, Classes, Globals, pdlstructs, patchconsts;

Type

  EPatchStructs = Class( Exception);
  EItemIndex    = Class( EPatchStructs);
  EPatch        = Class( EPatchStructs);

  TTColor = (
    coRED,
    coBLUE,
    coYELLOW,
    coGRAY,
    coGREEN,
    coPURPLE,
    coWHITE
  );

  TTConnector = (
    ctInput,
    ctOutput
  );

  TTModuleSection = (
    msCommon,
    msPoly,
    msMorph
  );

  TVoiceRetrigger = (
    vrInActive,
    vrAactive
  );

  TTMorph = (
    mtMorph1,
    mtMorph2,
    mtMorph3,
    mtMorph4
  );

  TKeyboardAssignment = (
    kaNone,
    kaVelocity,
    kaNote
  );

  TPortamento = (
    ptNormal,
    ptAuto
  );


  TCableVisibility = (
    cvVisible,
    cvInvisible
  );

  TRangeLimit = (
    rlMin,
    rlMax
  );

  TOnPatchError = Procedure( aSender: TObject; Const aMsg: String) Of Object;

  TIntIntMap = Class
  Private
    FData : Array Of Integer;
  Private
    Function    GetData( anIndex: Integer): Integer;
    Procedure   SetData( anIndex: Integer; aValue: Integer);
    Function    GetSize: Integer;
    Procedure   SetSize( aValue: Integer);
  Public
    Property    Data[ anIndex: Integer]: Integer Read GetData Write SetData; Default;
    Property    Size                   : Integer Read GetSize Write SetSize;
  End;

  TParameterMap = TIntIntMap;
  TCustomMap    = TIntIntMap;

  TNote     = Class;
  TKnobMap  = Class;
  TCtrlMap  = Class;
  TCable    = Class;
  TModule   = Class;
  TMorphMap = Class;

  TNoteList = Class( TPdlObjectList)
  Private
    Function    GetItem( anIndex: Integer): TNote;
  Public
    Procedure   Push( aValue: TNote);
    Function    Head: TNote;
    Procedure   WritePatch( Var aFile: TextFile);
  Public
    Property    Items[ anIndex: Integer]: TNote Read GetItem; Default;
  End;

  TKnobMapList = Class( TPdlObjectList)
  Private
    Function    GetItem( anIndex: Integer): TKnobMap;
  Public
    Procedure   Push( aValue: TKnobMap);
    Function    Head: TKnobMap;
  Public
    Property    Items[ anIndex: Integer]: TKnobMap Read GetItem; Default;
  End;

  TCtrlmapList = Class( TPdlObjectList)
  Private
    Function    GetItem( anIndex: Integer): TCtrlMap;
  Public
    Procedure   Push( aValue: TCtrlMap);
    Function    Head: TCtrlMap;
  Public
    Property    Items[ anIndex: Integer]: TCtrlMap Read GetItem; Default;
  End;

  TCableList = Class( TPdlObjectList)
  Private
    FModuleSection : TTModuleSection;
  Private
    Function    GetItem( anIndex: Integer): TCable;
  Public
    Constructor Create( aModuleSection: TTModuleSection);
    Procedure   Push( aValue: TCable);
    Function    Head: TCable;
  Public
    Property    Items[ anIndex: Integer]: TCable Read GetItem; Default;
  Public
    Property    ModuleSection : TTModuleSection Read FModuleSection;
  End;

  TModuleList = Class( TPdlObjectList)
  Private
    FModuleSection : TTModuleSection;
  Private
    Function    GetItem( anIndex: Integer): TModule;
  Public
    Constructor Create( aModuleSection: TTModuleSection);
    Procedure   Push( aValue: TModule);
    Function    Head: TModule;
  Public
    Property    Items[ anIndex: Integer]: TModule Read GetItem; Default;
  Public
    Property    ModuleSection : TTModuleSection Read FModuleSection;
  End;

  TMorphMapList = Class( TPdlObjectList)
  Private
    Function    GetItem( anIndex: Integer): TMorphMap;
  Public
    Procedure   Push( aValue: TMorphMap);
    Function    Head: TMorphMap;
    Procedure   WritePatch( Var aFile: TextFile);
  Public
    Property    Items[ anIndex: Integer]: TMorphMap Read GetItem; Default;
  End;

  TNote = Class
  Private
    FNoteNumber      : Integer;
    FAttackVelocity  : Integer;
    FReleaseVelocity : Integer;
  Public
    Constructor Create( aNoteNumber, anAtackVelocity, aReleaseVelocity: Integer);
  Public
    Property    NoteNumber     : Integer Read FNoteNumber;
    Property    AttackVelocity : Integer Read FAttackVelocity;
    Property    ReleaseVelocity: Integer Read FReleaseVelocity;
  End;

  TCable = Class
  Private
    FColor                    : TTColor;
    FDestinationModule        : TModule;
    FDestinationConnector     : Integer;
    FDestinationConnectorType : TTConnector;
    FSourceModule             : TModule;
    FSourceConnector          : Integer;
    FSourceConnectorType      : TTConnector;
  Public
    Constructor Create(
      aColor                    : TTColor;
      aDestinationModule        : TModule;
      aDestinationConnector     : Integer;
      aDestinationConnectorType : TTConnector;
      aSourceModule             : TModule;
      aSourceConnector          : Integer;
      aSourceConnectorType      : TTConnector
    );
    Procedure   WritePatch( Var aFile: TextFile);
  Public
    Property    Color                    : TTColor     Read FColor;
    Property    DestinationModule        : TModule     Read FDestinationModule;
    Property    DestinationConnector     : Integer     Read FDestinationConnector;
    Property    DestinationConnectorType : TTConnector Read FDestinationConnectorType;
    Property    SourceModule             : TModule     Read FSourceModule;
    Property    SourceConnector          : Integer     Read FSourceConnector;
    Property    SourceConnectorType      : TTConnector Read FSourceConnectorType;
  End;

  TCtrlMap = Class
  Private
    FModuleSection : TTModuleSection;
    FModule        : TModule;
    FParameter     : Integer;
    FCC            : Integer;
  Public
    Constructor Create(
      aModuleSection : TTModuleSection;
      aModule        : TModule;
      aParameter     : Integer
    );
    Procedure   WritePatch( Var aFile: TextFile);
  Public
    Property    ModuleSection : TTModuleSection Read FModuleSection;
    Property    Module        : TModule         Read FModule;
    Property    Parameter     : Integer         Read FParameter;
    Property    CC            : Integer         Read FCC            Write FCC;
  End;

  TKnobMap = Class
  Private
    FModuleSection : TTModuleSection;
    FModule        : TModule;
    FParameter     : Integer;
    FKnob          : Integer;
  Public
    Constructor Create(
      aModuleSection : TTModuleSection;
      aModule        : TModule;
      aParameter     : Integer
    );
    Procedure   WritePatch( Var aFile: TextFile);
  Public
    Property    ModuleSection : TTModuleSection Read FModuleSection;
    Property    Module        : TModule         Read FModule;
    Property    Parameter     : Integer         Read FParameter;
    Property    Knob          : Integer         Read FKnob          Write FKnob;
  End;

  TMorphMap = Class
  Private
    FIndex     : TTMorph;
    FSection   : TTModuleSection;
    FModule    : TModule;
    FParameter : Integer;
    FRange     : Integer;
  Public
    Constructor Create( anIndex: TTMorph; aSection: TTModuleSection; aModule: TModule; aParameter: Integer);
    Procedure   WritePatch( Var aFile: TextFile);
  Public
    Property    Index    : TTMorph         Read FIndex;
    Property    Section  : TTModuleSection Read FSection;
    Property    Parameter: Integer         Read FParameter;
    Property    Range    : Integer         Read FRange Write FRange;
  End;

  TModule = Class
  Private
    FIndex      : Integer;
    FModuleType : Integer;
    FXPos       : Integer;
    FYPos       : Integer;
    FName       : String;
    FParameters : TParameterMap;
    FCustoms    : TCustomMap;
  Private
    Function    GetParameterCount: Integer;
    Function    GetCustomCount   : Integer;
    Function    GetParameter( anIndex: Integer): Integer;
    Procedure   SetParameter( anIndex, aValue: Integer);
    Function    GetCustom( anIndex: Integer): Integer;
    Procedure   SetCustom( anIndex, aValue: Integer);
  Public
    Constructor Create( aModuleType, anIndex: Integer);
    Destructor  Destroy;                                               Override;
    Procedure   SetPosition( X, Y: Integer);
    Procedure   WritePatch( Var aFile: TextFile);
    Procedure   WriteParameters( Var aFile: TextFile);
    Procedure   WriteCustoms( Var aFile: TextFile);
  Public
    Property    Index          : Integer Read FIndex;
    Property    ModuleType     : Integer Read FModuleType;
    Property    XPosition      : Integer Read FXPos       Write FXpos;
    Property    YPosition      : Integer Read FYPos       Write FYPos;
    Property    Name           : String  Read FName       Write FName;
    Property    ParameterCount : Integer Read GetParameterCount;
    Property    CustomCount    : Integer Read GetCustomCount;
  Public
    Property    Parameter[ anIndex: Integer]: Integer Read GetParameter Write SetParameter;
    Property    Custom   [ anIndex: Integer]: Integer Read GetCustom    Write SetCustom;
  End;

  TModuleSection = Class
  Private
    FModuleSection  : TTModuleSection;
    FVoiceRetrigger : TVoiceRetrigger;
    FModules        : TModuleList;
    FCables         : TCableList;
    FNextIndex      : Integer;
  Private
    Function    GetModule( anIndex: Integer): TModule;
  Public
    Constructor Create( aSectionIndex: TTModuleSection);
    Destructor  Destroy;                                               Override;
    Function    NewModule( aModuleType: Integer; anIndex: Integer = 0): TModule;
    Procedure   RemoveModule( aModule: TModule);
    Function    NewCable(
      aColor                    : TTColor;
      aDestinationModuleIndex   : Integer;
      aDestinationConnector     : Integer;
      aDestinationConnectorType : TTConnector;
      aSourceModuleIndex        : Integer;
      aSourceConnector          : Integer;
      aSourceConnectorType      : TTConnector
    ): TCable;
    Procedure   RemoveCable( aCable: TCable);
    Procedure   WritePatchModules( Var aFile: TextFile);
    Procedure   WritePatchCables ( Var aFile: TextFile);
    Procedure   WriteParameters  ( Var aFile: TextFile);
    Procedure   WriteCustoms     ( Var aFile: TextFile);
    Procedure   WriteNames       ( Var aFile: TextFile);
  Public
    Property    VoiceRetrigger: TVoiceRetrigger Read FVoiceRetrigger Write FVoiceRetrigger;
    Property    Modules       : TModuleList     Read FModules;
    Property    Cables        : TCableList      Read FCables;
  Public
    Property    Module[ anIndex: Integer]: TModule Read GetModule; Default;
  Public
    Property    ModuleSection: TTModuleSection Read FModuleSection;
  End;

  TMorph = Class
  Private
    FIndex              : TTMorph;
    FValue              : Integer;
    FMorphMaps          : TMorphMapList;
    FKeyboardAssignment : TKeyboardAssignment;
  Public
    Constructor Create( anIndex: TTMorph);
    Destructor  Destroy;                                               Override;
    Function    NewMorphmap( aSection: TTModuleSection; aModule: TModule; aParameter: Integer): TMorphMap;
    Procedure   RemoveMorphMap( aMorphMap: TMorphMap);
    Procedure   WritePatch( Var aFile: TextFile);
  Public
    Property    MorphMaps         : TMorphMapList       Read FMorphMaps;
    Property    Value             : Integer             Read FValue              Write FValue;
    Property    KeyboardAssignment: TKeyboardAssignment Read FKeyboardAssignment Write FKeyboardAssignment;
  End;

  TPatch = Class
  Private
    FName                : String;
    FNotes               : TStringList;
    FPortamento          : TPortamento;
    FCableVisibility     : Array[ TTColor    ] Of Boolean;
    FKeyboardRange       : Array[ TRangeLimit] Of Integer;
    FVelocityRange       : Array[ TRangeLimit] Of Integer;
    FBendRange           : Integer;
    FPortamentoTime      : Integer;
    FRequestedVoices     : Integer;
    FSectionSeparatorPos : Integer;
    FOctaveShift         : Integer;
    FModuleSections      : Array[ TTModuleSection] Of TModuleSection;
    FMidiNotes           : TNoteList;
    FMorphs              : Array[ TTMorph] Of TMorph;
    FKnobMaps            : TKnobMapList;
    FCtrlMaps            : TCtrlMapList;
  Private
    FOnPatchError        : TOnPatchError;
  Private
    Procedure   SetNotes( aValue: TStringList);
  Private
    Function    GetCableVisibility( anIndex: TTColor): Boolean;
    Procedure   SetCableVisibility( anIndex: TTColor; aValue: Boolean);
    Function    GetKeyboardRange( anIndex: TRangeLimit): Integer;
    Procedure   SetKeyboardRange( anIndex: TRangeLimit; aValue: Integer);
    Function    GetVelocityRange( anIndex: TRangeLimit): Integer;
    Procedure   SetVelocityRange( anIndex: TRangeLimit; aValue: Integer);
    Function    GetModuleSection( anIndex: TTModuleSection): TModuleSection;
    Function    GetMorph( anIndex: TTMorph): TMorph;
  Protected
    Procedure   DoParseError( aSender: TObject; Const aMsg: String; Var AbortParsing: Boolean);
  Public
    Constructor Create;
    Procedure   Loadpatch( Const aName: String; DebugParser: Boolean = False);
    Destructor  Destroy;                                               Override;
    Function    NewMidiNote( aNoteNumber, anAttackVelocity, aReleaseVelocity: Integer): TNote;
    Procedure   RemoveMidiNote( aNote: TNote);
    Function    NewKnobMap(
      aModuleSection : TTModuleSection;
      aModule        : TModule;
      aParameter     : Integer
    ): TKnobMap;
    Procedure   RemoveKnobMap( aKnobMap: TKnobMap);
    Function    NewCtrlMap(
      aModuleSection : TTModuleSection;
      aModule        : TModule;
      aParameter     : Integer
    ): TCtrlMap;
    Procedure   RemoveCtrlMap( aCtrlMap: TCtrlMap);
    Procedure   WriteHeader            ( Var aFile: TextFile);
    Procedure   WriteKeyboardAssignment( Var aFile: TextFile);
    Procedure   WriteMorphMapDump      ( Var aFile: TextFile);
    Procedure   WriteKnobMapDump       ( Var aFile: TextFile);
    Procedure   WriteCtrlMapDump       ( Var aFile: TextFile);
    Procedure   WriteNotes             ( Var aFile: TextFile);
    Procedure   WritePatch( Var aFile: TextFile);                      Overload;
    Procedure   Writepatch( Const aFileName: String);                  Overload;
  Public
    Property    Name                 : String       Read FName                Write FName;
    Property    Notes                : TStringList  Read FNotes               Write SetNotes;
    Property    Portamento           : TPortamento  Read FPortamento          Write FPortamento;
    Property    BendRange            : Integer      Read FBendRange           Write FBendRange;
    Property    PortamentoTime       : Integer      Read FPortamentoTime      Write FPortamentoTime;
    Property    RequestedVoices      : Integer      Read FRequestedVoices     Write FRequestedVoices;
    Property    SectionSeparationPos : Integer      Read FSectionSeparatorPos Write FSectionSeparatorPos;
    Property    OctaveShift          : Integer      Read FOctaveShift         Write FOctaveShift;
    Property    MidiNotes            : TNoteList    Read FMidiNotes;
    Property    KnobMaps             : TKnobMapList Read FKnobMaps;
    Property    CtrlMaps             : TCtrlMapList Read FCtrlMaps;
  Public
    Property    CableVisibility[ anIndex: TTColor        ]: Boolean        Read GetCableVisibility Write SetCableVisibility;
    Property    KeyboardRange  [ anIndex: TRangeLimit    ]: Integer        Read GetKeyboardRange   Write SetkeyboardRange;
    Property    VelocityRange  [ anIndex: TRangeLimit    ]: Integer        Read GetVelocityRange   Write SetVelocityRange;
    Property    ModuleSection  [ anIndex: TTModuleSection]: TModuleSection Read GetModuleSection;
    Property    Morph          [ anIndex: TTMorph        ]: TMorph         Read GetMorph;
  Public
    Property    OnPatchError : TOnpatchError Read FOnPatchError Write FOnPatchError;
  End;


Implementation

Uses

  patchparser;
  

{ ========
  TIntIntMap = Class
  Private
    FData : Array Of Integer;
  Public
    Property    Data[ anIndex: Integer]: Integer Read GetData Write SetData; Default;
    Property    Size                   : Integer Read GetSize Write SetSize;
  Private
}

    Function    TIntIntMap.GetData( anIndex: Integer): Integer;
    Begin
      If ( anIndex >= 0) And ( anIndex < Size)
      Then Result := FData[ anIndex]
      Else Begin
        SetData( anIndex, -1);
        Result := -1;
      End;
    End;

    Procedure   TIntIntMap.SetData( anIndex: Integer; aValue: Integer);
    Begin
      If anIndex < 0
      Then Raise EItemIndex.Create( 'TIntIntMap : anIndex < 0 -> uselessss love, futile');
      If anIndex >= Size
      Then Size  := anIndex + 1;
      FData[ anIndex] := aValue;
    End;

    Function    TIntIntMap.GetSize: Integer;
    Begin
      Result := Length( FData);
    End;

    Procedure   TIntIntMap.SetSize( aValue: Integer);
    Begin
      SetLength( FData, aValue);
    End;

{ ========
  TNoteList = Class( TPdlObjectList)
  Public
    Property    Items[ anIndex: Integer]: TNote Read GetItem; Default;
  Private
}

    Function    TNoteList.GetItem( anIndex: Integer): TNote;
    Begin
      Result := Inherited Items[ anIndex] As TNote;
    End;

//  Public

    Procedure   TNoteList.Push( aValue: TNote);
    Begin
      Inherited Push( aValue);
    End;

    Function    TNoteList.Head: TNote;
    Begin
      Result := Inherited Head As TNote;
    End;

    Procedure   TNoteList.WritePatch( Var aFile: TextFile);
    Var
      i : Integer;
      S : String;
    Begin
      WriteLn( aFile, '[CurrentNoteDump]');
      S := '';
      For i := 0 To Size - 1 Do
        With Items[ i] Do
        Begin
          If S <> ''
          Then S := S + ' ';
          S := S + Format( '%d %d %d', [ NoteNumber, AttackVelocity, ReleaseVelocity]);
        End;
      If S <> ''
      Then WriteLn( aFile, S);
      WriteLn( aFile, '[/CurrentNoteDump]');
    End;


{ ========
  TKnobMapList = Class( TPdlObjectList)
  Public
    Property    Items[ anIndex: Integer]: TKnobMap Read GetItem; Default;
  Private
}

    Function    TKnobMapList.GetItem( anIndex: Integer): TKnobMap;
    Begin
      Result := Inherited Items[ anIndex] As TKnobMap;
    End;

//  Public

    Procedure   TKnobMapList.Push( aValue: TKnobMap);
    Begin
      Inherited Push( aValue);
    End;

    Function    TKnobMapList.Head: TKnobMap;
    Begin
      Result := Inherited Head As TKnobMap;
    End;


{ ========
  TCtrlmapList = Class( TPdlObjectList)
  Public
    Property    Items[ anIndex: Integer]: TCtrlMap Read GetItem; Default;
  Private
}

    Function    TCtrlMapList.GetItem( anIndex: Integer): TCtrlMap;
    Begin
      Result := Inherited Items[ anIndex] As TCtrlMap;
    End;

//  Public

    Procedure   TCtrlMapList.Push( aValue: TCtrlMap);
    Begin
      Inherited Push( aValue);
    End;

    Function    TCtrlMapList.Head: TCtrlMap;
    Begin
      Result := Inherited Head As TCtrlMap;
    End;


{ ========
  TCableList = Class( TPdlObjectList)
  Private
    FModuleSection : TTModuleSection;
  Public
    Property    Items[ anIndex: Integer]: TCable Read GetItem;
  Public
    Property    ModuleSection : TTModuleSection Read FModuleSection;
  Private
}

    Function    TCableList.GetItem( anIndex: Integer): TCable;
    Begin
      Result := Inherited Items[ anIndex] As TCable;
    End;

//  Public

    Constructor TCableList.Create( aModuleSection: TTModuleSection);
    Begin
      Inherited Create;
      FModuleSection := aModuleSection;
    End;

    Procedure   TCableList.Push( aValue: TCable);
    Begin
      Inherited Push( aValue);
    End;

    Function    TCableList.Head: TCable;
    Begin
      Result := Inherited Head As TCable;
    End;


{ ========
  TModuleList = Class( TPdlObjectList)
  Private
    FModuleSection : TTModuleSection;
  Public
    Property    Items[ anIndex: Integer]: TModule Read GetItem;
  Public
    Property    ModuleSection : TTModuleSection Read FModuleSection;
  Private
}

    Function    TModuleList.GetItem( anIndex: Integer): TModule;
    Begin
      Result := Inherited Items[ anIndex] As TModule;
    End;

//  Public

    Constructor TModuleList.Create( aModuleSection: TTModuleSection);
    Begin
      Inherited Create;
      FModuleSection := aModuleSection;
    End;

    Procedure   TModuleList.Push( aValue: TModule);
    Begin
      Inherited Push( aValue);
    End;

    Function    TModuleList.Head: TModule;
    Begin
      Result := Inherited Head As TModule;
    End;


{ ========
  TMorphMapList = Class( TPdlObjectList)
  Public
    Property    Items[ anIndex: Integer]: TMorphMap Read GetItem; Default;
  Private
}

    Function    TMorphMapList.GetItem( anIndex: Integer): TMorphMap;
    Begin
      Result := Inherited Items[ anIndex] As TMorphMap;
    End;

//  Public

    Procedure   TMorphMapList.Push( aValue: TMorphMap);
    Begin
      Inherited Push( aValue);
    End;

    Function    TMorphMapList.Head: TMorphMap;
    Begin
      Result := Inherited Head As TMorphMap;
    End;

    Procedure   TMorphMapList.WritePatch( Var aFile: TextFile);
    Var
      i : Integer;
    Begin
      For i := 0 To Size - 1 Do
        Items[ i].WritePatch( aFile);
    End;


{ ========
  TNote = Class
  Private
    FNoteNumber      : Integer;
    FAttackVelocity  : Integer;
    FReleaseVelocity : Integer;
  Public
    Property    NoteNumber     : Integer Read FNoteNumber;
    Property    AttackVelocity : Integer Read FAttackVelocity;
    Property    ReleaseVelocity: Integer Read FReleaseVelocity;
  Public
}

    Constructor TNote.Create( aNoteNumber, anAtackVelocity, aReleaseVelocity: Integer);
    Begin
      Inherited Create;
      FNoteNumber      := aNoteNumber;
      FAttackVelocity  := anAtackVelocity;
      FReleaseVelocity := aReleaseVelocity;
    End;


{ ========
  TCable = Class
  Private
    FColor                    : TTColor;
    FDestinationModule        : TModule;
    FDestinationConnector     : Integer;
    FDestinationConnectorType : TTConnector;
    FSourceModule             : TModule;
    FSourceConnector          : Integer;
    FSourceConnectorType      : TTConnector;
  Public
    Property    Color                    : TTColor     Read FColor;
    Property    DestinationModule        : TModule     Read FDestinationModule;
    Property    DestinationConnector     : Integer     Read FDestinationConnector;
    Property    DestinationConnectorType : TTConnector Read FDestinationConnectorType;
    Property    SourceModule             : TModule     Read FSourceModule;
    Property    SourceConnector          : Integer     Read FSourceConnector;
    Property    SourceConnectorType      : TTConnector Read FSourceConnectorType;
  Public
}

    Constructor TCable.Create(
      aColor                    : TTColor;
      aDestinationModule        : TModule;
      aDestinationConnector     : Integer;
      aDestinationConnectorType : TTConnector;
      aSourceModule             : TModule;
      aSourceConnector          : Integer;
      aSourceConnectorType      : TTConnector
    );
    Begin
      Inherited Create;
      FColor                    := aColor;
      FDestinationModule        := aDestinationModule;
      FDestinationConnector     := aDestinationConnector;
      FDestinationConnectorType := aDestinationConnectorType;
      FSourceModule             := aSourceModule;
      FSourceConnector          := aSourceConnector;
      FSourceConnectorType      := aSourceConnectorType;
    End;

    Procedure   TCable.WritePatch( Var aFile: TextFile);
    Begin
      WriteLn(
        aFile,
        Format(
          '%d %d %d %d %d %d %d',
          [
            Ord( Color),
            DestinationModule.Index,
            DestinationConnector,
            Ord( DestinationConnectorType),
            SourceModule.Index,
            SourceConnector,
            Ord( SourceConnectorType)
          ]
        )
      );
    End;
    

{ ========
  TCtrlMap = Class
  Private
    FModuleSection : TTModuleSection;
    FModule        : TModule;
    FParameter     : Integer;
    FCC            : Integer;
  Public
    Property    ModuleSection : TTModuleSection Read FModuleSection;
    Property    Module        : TModule         Read FModule;
    Property    Parameter     : Integer         Read FParameter;
    Property    CC            : Integer         Read FCC            Write FCC;
  Public
}

    Constructor TCtrlMap.Create(
      aModuleSection : TTModuleSection;
      aModule        : TModule;
      aParameter     : Integer
    );
    Begin
      FModuleSection := aModuleSection;
      FModule        := aModule;
      FParameter     := aParameter;
      Fcc            := -1;
    End;

    Procedure   TCtrlMap.WritePatch( Var aFile: TextFile);
    Begin
      If Assigned( Module)
      Then WriteLn( aFile, Format( '%d %d %d %d', [ Ord( ModuleSection), Module.Index, Parameter, CC]))
      Else WriteLn( aFile, Format( '%d %d %d %d', [ 2                  , 1           , Parameter, CC]));
    End;


{ ========
  TKnobMap = Class
  Private
    FModuleSection : TTModuleSection;
    FModule        : TModule;
    FParameter     : Integer;
    FKnob          : Integer;
  Public
    Property    ModuleSection : TTModuleSection Read FModuleSection;
    Property    Module        : TModule         Read FModule;
    Property    Parameter     : Integer         Read FParameter;
    Property    Knob          : Integer         Read FKnob          Write FKnob;
  Public
}

    Constructor TKnobMap.Create(
      aModuleSection : TTModuleSection;
      aModule        : TModule;
      aParameter     : Integer
    );
    Begin
      FModuleSection := aModuleSection;
      FModule        := aModule;
      FParameter     := aParameter;
      FKnob          := -1;
    End;

    Procedure   TKnobMap.WritePatch( Var aFile: TextFile);
    Begin
      If Assigned( Module)
      Then WriteLn( aFile, Format( '%d %d %d %d', [ Ord( ModuleSection), Module.Index, Parameter, Knob]))
      Else WriteLn( aFile, Format( '2 1 %d %d', [ Parameter, Knob]));
    End;


{ ========
  TMorphMap = Class
  Private
    FIndex     : TTMorph;
    FSection   : TTModuleSection;
    FModule    : TModule;
    FParameter : Integer;
    FRange     : Integer;
  Public
    Property    Index    : TTMorph         Read FIndex;
    Property    Section  : TTModuleSection Read FSection;
    Property    Parameter: Integer         Read FParameter;
    Property    Range    : Integer         Read FRange Write FRange;
  Public
}

    Constructor TMorphMap.Create( anIndex: TTMorph; aSection: TTModuleSection; aModule: TModule; aParameter: Integer);
    Begin
      Inherited Create;
      FIndex     := anIndex;
      FSection   := aSection;
      FModule    := aModule;
      Fparameter := aParameter;
    End;

    Procedure   TMorphMap.WritePatch( Var aFile: TextFile);
    Begin
      If Assigned( FModule)
      Then
        WriteLn(
          aFile,
          Format(
            '%d %d %d %d %d',
            [
              Ord( Section),
              FModule.Index,
              FParameter,
              Ord( Index),
              FRange
            ]
          )
        )
      Else
        WriteLn(
          aFile,
          Format(
            '2 1 %d %d %d',
            [
              FParameter,
              Ord( Index),
              FRange
            ]
          )
        )
    End;


{ ========
  TModule = Class
  Private
    FModuleType : Integer;
    FIndex      : Integer;
    FXPos       : Integer;
    FYPos       : Integer;
    FName       : String;
    FParameters : TParameterMap;
    FCustoms    : TCustomMap;
  Public
    Property    Index          : Integer Read FIndex;
    Property    ModuleType     : Integer Read FModuleType;
    Property    XPosition      : Integer Read FXPos       Write FXpos;
    Property    YPosition      : Integer Read FYPos       Write FYPos;
    Property    Name           : String  Read FName       Write FName;
    Property    ParameterCount : Integer Read GetParameterCount;
    Property    CustomCount    : Integer Read GetCustomCount;
  Public
    Property    Parameter[ anIndex: Integer]: Integer Read GetParameter Write SetParameter;
    Property    Custom   [ anIndex: Integer]: Integer Read GetCustom    Write SetCustom;
  Private
}

    Function    TModule.GetParameterCount: Integer;
    Begin
      Result := FParameters.Size;
    End;

    Function    TModule.GetCustomCount : Integer;
    Begin
      Result := FCustoms.Size;
    End;

    Function    TModule.GetParameter( anIndex: Integer): Integer;
    Begin
      Result := FParameters[ anIndex]
    End;

    Procedure   TModule.SetParameter( anIndex, aValue: Integer);
    Begin
      FParameters[ anIndex] := aValue;
    End;

    Function    TModule.GetCustom( anIndex: Integer): Integer;
    Begin
      Result := FCustoms[ anIndex]
    End;

    Procedure   TModule.SetCustom( anIndex, aValue: Integer);
    Begin
      FCustoms[ anIndex] := aValue;
    End;

//  Public

    Constructor TModule.Create( aModuleType, anIndex: Integer);
    Begin
      Inherited Create;
      FParameters := TParameterMap.Create;
      FCustoms    := TCustomMap   .Create;
      FModuleType := aModuleType;
      FIndex      := anIndex;
    End;

    Destructor  TModule.Destroy; // Override;
    Begin
      FreeAndNil( FCustoms   );
      FreeAndNil( FParameters);
      Inherited;
    End;

    Procedure   TModule.SetPosition( X, Y: Integer);
    Begin
      XPosition := X;
      YPosition := Y;
    End;

    Procedure   TModule.WritePatch( Var aFile: TextFile);
    Begin
      WriteLn( aFile, Format( '%d %d %d %d', [ Index, ModuleType, XPosition, YPosition]));
    End;

    Procedure   TModule.WriteParameters( Var aFile: TextFile);
    Var
      i : Integer;
      S : String;
    Begin
      If parameterCount > 0
      Then Begin
        S := Format( '%d %d %d', [ Index, ModuleType, ParameterCount]);
        For i := 0 To FParameters.Size - 1 Do
          S := S + Format( ' %d', [ FParameters[ i]]);
        WriteLn( aFile, S);
      End;
    End;

    Procedure   TModule.WriteCustoms( Var aFile: TextFile);
    Var
      i : Integer;
      S : String;
    Begin
      If CustomCount > 0
      Then Begin
        S := Format( '%d %d', [ Index, CustomCount]);
        For i := 0 To FCustoms.Size - 1 Do
          S := S + Format( ' %d', [ FCustoms[ i]]);
        WriteLn( aFile, S);
      End;
    End;


{ ========
  TModuleSection = Class
  Private
    FModuleSection  : TTModuleSection;
    FVoiceRetrigger : TVoiceRetrigger;
    FModules        : TModuleList;
    FCables         : TCableList;
    FNextIndex      : Integer;
  Public
    Property    VoiceRetrigger: TVoiceRetrigger Read FVoiceRetrigger Write FVoiceRetrigger;
    Property    Modules       : TModuleList     Read FModules;
    Property    Cables        : TCableList      Read FCables;
  Public
    Property    Module[ anIndex: Integer]: TModule Read GetModule; Default;
  Public
    Property    ModuleSection: TTModuleSection Read FModuleSection;
  Private
}

    Function    TModuleSection.GetModule( anIndex: Integer): TModule;
    Var
      i : Integer;
    Begin
      Result := Nil;
      With FModules Do
        For i:= 0 To Size - 1 Do
          If FModules[ i].Index = anIndex
          Then Begin
            Result := FModules[ i];
            Break;
          End;
    End;

//  Public

    Constructor TModuleSection.Create( aSectionIndex: TTModuleSection);
    Begin
      Inherited Create;
      FModules := TModuleList.Create( aSectionIndex);
      FCables  := TCableList .Create( aSectionIndex);
      FModuleSection := aSectionIndex;
    End;

    Destructor  TModuleSection.Destroy; // Override;
    Begin
      FreeAndNil( FModules); // Frees the elements as well
      FreeAndNil( FCables ); // Frees the elements as well
      Inherited;
    End;

    Function    TModuleSection.NewModule( aModuleType: Integer; anIndex: Integer = 0): TModule;
    Begin
      If anIndex = 0
      Then Begin
        anIndex := FNextIndex;
        Inc( FNextIndex);
      End
      Else Begin
        If FNextIndex <= anIndex + 1
        Then FNextIndex := anIndex + 1;
      End;
      Result := TModule.Create( aModuleType, anIndex);
      FModules.Push( Result);
    End;

    Procedure   TModuleSection.RemoveModule( aModule: TModule);
    Begin
      FModules.Remove( aModule);
      If Assigned( aModule)
      Then aModule.Free;
    End;

    Function    TModuleSection.NewCable(
      aColor                    : TTColor;
      aDestinationModuleIndex   : Integer;
      aDestinationConnector     : Integer;
      aDestinationConnectorType : TTConnector;
      aSourceModuleIndex        : Integer;
      aSourceConnector          : Integer;
      aSourceConnectorType      : TTConnector
    ): TCable;
    Begin
      Result :=
        TCable.
          Create(
            aColor,
            Module[ aDestinationModuleIndex],
            aDestinationConnector,
            aDestinationConnectorType,
            Module[ aSourceModuleIndex],
            aSourceConnector,
            aSourceConnectorType
          );
      Cables.Push( Result);
    End;

    Procedure   TModuleSection.RemoveCable( aCable: TCable);
    Begin
      FCables.Remove( aCable);
      If Assigned( aCable)
      Then aCable.Free;
    End;

    Procedure   TModuleSection.WritePatchModules( Var aFile: TextFile);
    Var
      i : Integer;
    Begin
      WriteLn( aFile, '[ModuleDump]');
      WriteLn( aFile, Format( '%d', [ Ord( ModuleSection)]));
      For i := 0 To Modules.Size - 1 Do
        FModules[ i].WritePatch( aFile);
      WriteLn( aFile, '[/ModuleDump]');
    End;

    Procedure   TModuleSection.WritePatchCables( Var aFile: TextFile);
    Var
      i : Integer;
    Begin
      WriteLn( aFile, '[CableDump]');
      WriteLn( aFile, Format( '%d', [ Ord( ModuleSection)]));
      For i := 0 To Cables.Size - 1 Do
        Cables[ i].WritePatch( aFile);
      WriteLn( aFile, '[/CableDump]');
    End;

    Procedure   TModuleSection.WriteParameters( Var aFile: TextFile);
    Var
      i : Integer;
    Begin
      WriteLn( aFile, '[ParameterDump]');
      WriteLn( aFile, Format( '%d', [ Ord( ModuleSection)]));
      For i := 0 To Modules.Size - 1 Do
        Modules[ i].WriteParameters( aFile);
      WriteLn( aFile, '[/ParameterDump]');
    End;

    Procedure   TModuleSection.WriteCustoms( Var aFile: TextFile);
    Var
      i : Integer;
    Begin
      WriteLn( aFile, '[CustomDump]');
      WriteLn( aFile, Format( '%d', [ Ord( ModuleSection)]));
      For i := 0 To Modules.Size - 1 Do
        Modules[ i].WriteCustoms( aFile);
      WriteLn( aFile, '[/CustomDump]');
    End;

    Procedure   TModuleSection.WriteNames( Var aFile: TextFile);
    Var
      i : Integer;
    Begin
      WriteLn( aFile, '[NameDump]');
      WriteLn( aFile, Format( '%d', [ Ord( ModuleSection)]));
      For i := 0 To Modules.Size - 1 Do
        With Modules[ i] Do
          WriteLn( aFile, Format( '%d %s', [ Index, Name]));
      WriteLn( aFile, '[/NameDump]');
    End;
    

{ ========
  TMorph = Class
  Private
    FIndex              : TTMorph;
    FValue              : Integer;
    FMorphMaps          : TMorphMapList;
    FKeyboardAssignment : TKeyboardAssignment;
  Public
    Property    MorphMaps         : TMorphMapList       Read FMorphMaps;
    Property    Value             : Integer             Read FValue              Write FValue;
    Property    KeyboardAssignment: TKeyboardAssignment Read FKeyboardAssignment Write FKeyboardAssignment;
  Public
}

    Constructor TMorph.Create( anIndex: TTMorph);
    Begin
      Inherited Create;
      FIndex     := anIndex;
      FMorphMaps := TMorphMapList.Create;
    End;

    Destructor  TMorph.Destroy; // Override;
    Begin
      FreeAndNil( FMorphMaps);
      Inherited;
    End;

    Function    TMorph.NewMorphmap( aSection: TTModuleSection; aModule: TModule; aParameter: Integer): TMorphMap;
    Begin
      Result := TMorphMap.Create( FIndex, aSection, aModule, aParameter);
      FMorphMaps.Push( Result);
    End;

    Procedure   TMorph.RemoveMorphMap( aMorphMap: TMorphMap);
    Begin
      FMorphMaps.Remove( aMorphMap);
      If Assigned( aMorphMap)
      Then aMorphMap.Free;
    End;

    Procedure   TMorph.WritePatch( Var aFile: TextFile);
    Var
      i : Integer;
    Begin
      With MorphMaps Do
        For i := 0 To Size - 1 Do
          WritePatch( aFile);
    End;


{ ========
  TPatch = Class
  Private
    FName                : String;
    FNotes               : TStringList;
    FPortamento          : TPortamento;
    FCableVisibility     : Array[ TTColor    ] Of Boolean;
    FKeyboardRange       : Array[ TRangeLimit] Of Integer;
    FVelocityRange       : Array[ TRangeLimit] Of Integer;
    FBendRange           : Integer;
    FPortamentoTime      : Integer;
    FRequestedVoices     : Integer;
    FSectionSeparatorPos : Integer;
    FOctaveShift         : Integer;
    FModuleSections      : Array[ TTModuleSection] Of TModuleSection;
    FMidiNotes           : TNoteList;
    FMorphs              : Array[ TTMorph] Of TMorph;
    FKnobMaps            : TKnobMapList;
    FCtrlMaps            : TCtrlMapList;
  Public
    Property    Name                 : String       Read FName                Write FName;
    Property    Notes                : TStringList  Read FNotes               Write SetNotes;
    Property    Portamento           : TPortamento  Read FPortamento          Write FPortamento;
    Property    BendRange            : Integer      Read FBendRange           Write FBendRange;
    Property    PortamentoTime       : Integer      Read FPortamentoTime      Write FPortamentoTime;
    Property    RequestedVoices      : Integer      Read FRequestedVoices     Write FRequestedVoices;
    Property    SectionSeparationPos : Integer      Read FSectionSeparatorPos Write FSectionSeparatorPos;
    Property    OctaveShift          : Integer      Read FOctaveShift         Write FOctaveShift;
    Property    MidiNotes            : TNoteList    Read FMidiNotes;
    Property    KnobMaps             : TKnobMapList Read FKnobMaps;
    Property    CtrlMaps             : TCtrlMapList Read FCtrlMaps;
  Public
    Property    CableVisibility[ anIndex: TTColor        ]: Boolean        Read GetCableVisibility Write SetCableVisibility;
    Property    KeyboardRange  [ anIndex: TRangeLimit    ]: Integer        Read GetKeyboardRange   Write SetkeyboardRange;
    Property    VelocityRange  [ anIndex: TRangeLimit    ]: Integer        Read GetVelocityRange   Write SetVelocityRange;
    Property    ModuleSection  [ anIndex: TTModuleSection]: TModuleSection Read GetModuleSection;
    Property    Morph          [ anIndex: TTMorph        ]: TMorph         Read GetMorph;
  Public
    Property    OnPatchError : TOnpatchError Read FOnPatchError Write FOnPatchError;
  Private
}

    Procedure   TPatch.SetNotes( aValue: TStringList);
    Begin
      FNotes.Assign( aValue);
    End;

//  Private

    Function    TPatch.GetCableVisibility( anIndex: TTColor): Boolean;
    Begin
      Result := FCableVisibility[ anIndex];
    End;

    Procedure   TPatch.SetCableVisibility( anIndex: TTColor; aValue: Boolean);
    Begin
      FCableVisibility[ anIndex] := aValue;
    End;

    Function    TPatch.GetKeyboardRange( anIndex: TRangeLimit): Integer;
    Begin
      Result := FKeyboardRange[ anIndex];
    End;

    Procedure   TPatch.SetKeyboardRange( anIndex: TRangeLimit; aValue: Integer);
    Begin
      FKeyboardRange[ anIndex] := aValue;
    End;

    Function    TPatch.GetVelocityRange( anIndex: TRangeLimit): Integer;
    Begin
      Result := FVelocityRange[ anIndex];
    End;

    Procedure   TPatch.SetVelocityRange( anIndex: TRangeLimit; aValue: Integer);
    Begin
      FVelocityRange[ anIndex] := aValue;
    End;

    Function    TPatch.GetModuleSection( anIndex: TTModuleSection): TModuleSection;
    Begin
      Result := FModuleSections[ anIndex];
    End;

    Function    TPatch.GetMorph( anIndex: TTMorph): TMorph;
    Begin
      Result := FMorphs[ anIndex];
    End;

//  Protected

    Procedure   TPatch.DoParseError( aSender: TObject; Const aMsg: String; Var AbortParsing: Boolean);
    Begin
      AbortParsing := False;            
      If Assigned( FOnPatchError)
      Then FOnPatchError( Self, aMsg)
      Else Raise EPatch.Create( aMsg);
    End;

//  Public

    Constructor TPatch.Create; 
    Begin
      Inherited;
      FNotes     := TStringlist.Create;
      FMidiNotes := TNoteList.Create;
      FKnobMaps  := TKnobMapList.Create;
      FCtrlMaps  := TCtrlmapList.Create;
      FModuleSections[ msPoly  ] := TModuleSection.Create( msPoly  );
      FModuleSections[ msCommon] := TModuleSection.Create( msCommon);
      FMorphs[ mtMorph1] := TMorph.Create( mtMorph1);
      FMorphs[ mtMorph2] := TMorph.Create( mtMorph2);
      FMorphs[ mtMorph3] := TMorph.Create( mtMorph3);
      FMorphs[ mtMorph4] := TMorph.Create( mtMorph4);
    End;

    Procedure   TPatch.LoadPatch( Const aName: String; DebugParser: Boolean = False);
    Var
      aParser      : TParser;
      AbortParsing : Boolean;
    Begin
      Name := CleanFileName( aName);
      aParser := TParser.Create;
      InitPatch( Self);
      With aParser Do
      Try
        Debug        := Debugparser;
        OnParseError := DoParseError;
        PushFile( aName);
        AbortParsing := False;
        If ( Parse <> 0) Or ( ErrorCount <> 0)
        Then DoParseError( Self, 'Errors', AbortParsing)
        // Else DoParseError( Self, 'Ok.'   , AbortParsing);
      Finally
        Free;
      End;
    End;

    Destructor  TPatch.Destroy; // Override;
    Begin
      FreeAndNil( FMorphs[ mtMorph4]);
      FreeAndNil( FMorphs[ mtMorph3]);
      FreeAndNil( FMorphs[ mtMorph2]);
      FreeAndNil( FMorphs[ mtMorph1]);
      FreeAndNil( FModuleSections[ msCommon]);
      FreeAndNil( FModuleSections[ msPoly  ]);
      FreeAndNil( FCtrlMaps);
      FreeAndNil( FKnobMaps);
      FreeAndNil( FMidiNotes);
      FreeAndNil( FNotes);
      Inherited;
    End;

    Function    TPatch.NewMidiNote( aNoteNumber, anAttackVelocity, aReleaseVelocity: Integer): TNote;
    Begin
      Result := TNote.Create( aNoteNumber, anAttackVelocity, aReleaseVelocity);
      FMidiNotes.Push( Result);
    End;

    Procedure   TPatch.RemoveMidiNote( aNote: TNote);
    Begin
      FMidiNotes.Remove( aNote);
      If Assigned( aNote)
      Then aNote.Free;
    End;

    Function    TPatch.NewKnobMap(
      aModuleSection : TTModuleSection;
      aModule        : TModule;
      aParameter     : Integer
    ): TKnobMap;
    Begin
      Result := TKnobMap.Create( aModuleSection, aModule, aParameter);
      FKnobMaps.Push( Result);
    End;

    Procedure   TPatch.RemoveKnobMap( aKnobMap: TKnobMap);
    Begin
      FKnobMaps.Remove( aKnobMap);
      If Assigned( aKnobMap)
      Then aKnobMap.Free;
    End;

    Function    TPatch.NewCtrlMap(
      aModuleSection : TTModuleSection;
      aModule        : TModule;
      aParameter     : Integer
    ): TCtrlMap;
    Begin
      Result := TCtrlMap.Create( aModuleSection, aModule, aParameter);
      FCtrlmaps.Push( Result);
    End;

    Procedure   TPatch.RemoveCtrlMap( aCtrlMap: TCtrlMap);
    Begin
      FCtrlMaps.Remove( aCtrlMap);
      If Assigned( aCtrlMap)
      Then aCtrlMap.Free;
    End;

    Procedure   TPatch.WriteHeader( Var aFile: TextFile);
    Begin
      WriteLn( aFile, '[Header]');
      WriteLn( aFile, 'Version=Nord Modular patch 3.0');
      WriteLn(
        aFile,
        Format(
          '%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d',
          [
            KeyboardRange[ rlMin],
            KeyboardRange[ rlMax],
            VelocityRange[ rlMin],
            VelocityRange[ rlMax],
            BendRange,
            PortamentoTime,
            Ord( Portamento),
            RequestedVoices,
            SectionSeparationPos,
            OctaveShift,
            Ord( ModuleSection[ msPoly  ].VoiceRetrigger),
            Ord( ModuleSection[ msCommon].VoiceRetrigger),
            1,
            1,
            1,
            1,
            Ord( CableVisibility[ coRED   ]),
            Ord( CableVisibility[ coBLUE  ]),
            Ord( CableVisibility[ coYELLOW]),
            Ord( CableVisibility[ coGRAY  ]),
            Ord( CableVisibility[ coGREEN ]),
            Ord( CableVisibility[ coPURPLE]),
            Ord( CableVisibility[ coWHITE ])
          ]
        )
      );
      WriteLn( aFile, '[/Header]');
    End;

    Procedure   TPatch.WriteKeyboardAssignment( Var aFile : TextFile);
    Var
      m    : TTMorph;
      Doit : Boolean;
      S    : String;
    Begin
      Doit := False;
      For m := Low( FMorphs) To High( FMorphs) Do
        If FMorphs[ m].KeyboardAssignment <> kaNone
        Then Begin
          Doit := True;
          Break;
        End;
      If Doit
      Then Begin
        WriteLn( aFile, '[KeyboardAssignment]');
        S := '';
        For m := Low( FMorphs) To High( FMorphs) Do
          S := S + Format( '%d ', [ Ord( FMorphs[ m].KeyboardAssignment)]);
        S := Copy( S, 1, Length( S) - 1);
        WriteLn( aFile, S);
        WriteLn( aFile, '[/KeyboardAssignment]');
      End;
    End;

    Procedure   TPatch.WriteMorphMapDump( Var aFile : TextFile);
    Var
      m    : TTMorph;
      S    : String;
      Doit : Boolean;
    Begin
      DoIt := False;
      For m := Low( FMorphs) To High( FMorphs) Do
        If FMorphs[ m].MorphMaps.Size > 0
        Then Begin
          DoIt := True;
          Break;
        End;
      If DoIt
      Then Begin
        WriteLn( aFile, '[MorphMapDump]');
        S := '';
        For m := Low( FMorphs) To High( FMorphs) Do
          S := S + Format( '%d ', [ FMorphs[ m].Value]);
        S := Copy( S, 1, Length( S) - 1);
        WriteLn( aFile, S);
        For m := Low( FMorphs) To High( FMorphs) Do
          FMorphs[ m].WritePatch( aFile);
        WriteLn( aFile, '[/MorphMapDump]');
      End;
    End;

    Procedure   TPatch.WriteKnobMapDump( Var aFile : TextFile);
    Var
      i : Integer;
    Begin
      With KnobMaps Do
        If Size > 0
        Then Begin
          WriteLn( aFile, '[KnobMapDump]');
          For i := 0 To Size - 1 Do
            If Assigned( Items[ i])
            Then Items[ i].WritePatch( aFile);
          WriteLn( aFile, '[/KnobMapDump]');
        End;
    End;

    Procedure   TPatch.WriteCtrlMapDump( Var aFile : TextFile);
    Var
      i : Integer;
    Begin
      With CtrlMaps Do
        If Size > 0
        Then Begin
          WriteLn( aFile, '[CtrlMapDump]');
          For i := 0 To Size - 1 Do
            Items[ i].WritePatch( aFile);
          WriteLn( aFile, '[/CtrlMapDump]');
        End;
    End;

    Procedure   TPatch.WriteNotes( Var aFile: TextFile);
    Var
      i : Integer;
    Begin
      With FNotes Do
      Begin
        If Count > 0
        Then begin
          WriteLn( aFile, '[Notes]');
          For i := 0 To Count - 1 Do
            WriteLn( aFile, Strings[ i]);
          WriteLn( aFile, '[/Notes]');
        End;
      End;
    End;

    Procedure   TPatch.WritePatch( Var aFile: TextFile); // Overload;
    Begin
      WriteHeader( aFile);

      ModuleSection[ msPoly  ].WritePatchModules( aFile);
      ModuleSection[ msCommon].WritePatchModules( aFile);

      MidiNotes.WritePatch( aFile);

      ModuleSection[ msPoly  ].WritePatchCables( aFile);
      ModuleSection[ msCommon].WritePatchCables( aFile);

      ModuleSection[ msPoly  ].WriteParameters( aFile);
      ModuleSection[ msCommon].WriteParameters( aFile);

      WriteKnobMapDump( aFile);
      WriteCtrlMapDump( aFile);

      ModuleSection[ msPoly  ].WriteCustoms( aFile);
      ModuleSection[ msCommon].WriteCustoms( aFile);

      WriteKeyboardAssignment( aFile);
      WriteMorphMapDump      ( aFile);

      ModuleSection[ msPoly  ].WriteNames( aFile);
      ModuleSection[ msCommon].WriteNames( aFile);

      WriteNotes( aFile);
    End;

    Procedure   TPatch.WritePatch( Const aFileName: String); // Overload;
    Var
      aFile : TextFile;
    Begin
      AssignFile( aFile, aFileName);
      Rewrite( aFile);
      Try
        WritePatch( aFile);
      Finally
        CloseFile( aFile);
      End;
    End;



End.
