Unit FrmMain;

//  ////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2003 Jan Punter
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  ////////////////////////////////////////////////////////////////////////////
//
//    lupd : 2003-10-13
//
//  ////////////////////////////////////////////////////////////////////////////


Interface

Uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, Inifiles, ScktComp,

  Globals, pdlstructs, patchstructs, protocolstructs;

Type

  TFormMain = Class(TForm)
    PanelTop: TPanel;
    Label1: TLabel;
    EditTestFileName: TEdit;
    BitBtnBrowseTest: TBitBtn;
    BitBtnBeamIt: TBitBtn;
    BitBtnExit: TBitBtn;
    PanelMain: TPanel;
    MemoResults: TMemo;
    OpenDialogTest: TOpenDialog;
    ClientSocket: TClientSocket;
    TimerBeat: TTimer;
    BitBtnRunStop: TBitBtn;
    EditHostAddress: TEdit;
    EditHostPort: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    BitBtnConnect: TBitBtn;
    ImageRun: TImage;
    CheckBoxAutoStart: TCheckBox;
    CheckBoxAutoConnect: TCheckBox;
    SpeedButtonSlotA: TSpeedButton;
    SpeedButtonSlotB: TSpeedButton;
    SpeedButtonSlotD: TSpeedButton;
    SpeedButtonSlotC: TSpeedButton;
    BitBtnCls: TBitBtn;
    BitBtnSendIAm: TBitBtn;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BitBtnExitClick(Sender: TObject);
    procedure BitBtnBrowseTestClick(Sender: TObject);
    procedure BitBtnBeamItClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TimerBeatTimer(Sender: TObject);
    procedure BitBtnRunStopClick(Sender: TObject);
    procedure ClientSocketConnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientSocketDisconnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientSocketError(Sender: TObject; Socket: TCustomWinSocket;
      ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure ClientSocketLookup(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientSocketRead(Sender: TObject; Socket: TCustomWinSocket);
    procedure BitBtnConnectClick(Sender: TObject);
    procedure EditHostAddressChange(Sender: TObject);
    procedure EditHostPortChange(Sender: TObject);
    procedure CheckBoxAutoStartClick(Sender: TObject);
    procedure CheckBoxAutoConnectClick(Sender: TObject);
    procedure SpeedButtonSlotClick(Sender: TObject);
    procedure BitBtnClsClick(Sender: TObject);
    procedure BitBtnSendIAmClick(Sender: TObject);
  Private
    FTestFileName     : String;
    FNmProtocolDriver : TNMProtocolDriver;
    FPatch            : TPatch;
    FBeatState        : Integer;
    FConnected        : Boolean;
    FRunning          : Boolean;
    FHostAddress      : String;
    FHostPort         : String;
    FAutoStart        : Boolean;
    FAutoConnect      : Boolean;
    FStopping         : Boolean;
    FSlotSelection    : Array[ 0 .. 3] Of Boolean;
    FActiveSlot       : Integer;
  Private
    Procedure   SetTestFileName( Const aValue: String);
    Procedure   SetConnected( aValue: Boolean);
    Procedure   SetRunning  ( avalue: Boolean);
    Procedure   SetHostAddress( Const aValue: String);
    Procedure   SetHostPort   ( Const aValue: String);
    Procedure   SetAutoStart  ( aValue: Boolean);
    Procedure   SetAutoConnect( aValue: Boolean);
    Procedure   SetActiveSlot ( aValue: Integer);
  Private
    Function    GetSlotSelection( anIndex: Integer): Boolean;
    Procedure   SetSlotSelection( anIndex: Integer; aValue: Boolean);
  Private
    Procedure   LoadIni;
    Procedure   SaveIni;
  Private
    Procedure   BrowseForTestFile;
    Procedure   SendIAm;
    Procedure   SendActivateSlot( aTag: Integer;  aDown: Boolean);
    Procedure   SendPatch;
    Procedure   DoDriverData ( aSender: TObject; aData: TByteList);
    Procedure   DoMidiMessage( aSender: TObject; aData: TMidiMessage);
    Procedure   Log( Const aMsg: String);
    Procedure   LogFmt( Const aFormat: String; Const anArgs: Array Of Const);
  Private
    Property    TestFileName: String  Read FTestFileName Write SetTestFileName;
    Property    Connected   : Boolean Read FConnected    Write SetConnected;
    Property    Running     : Boolean Read FRunning      Write SetRunning;
    Property    HostAddress : String  Read FHostAddress  Write SetHostAddress;
    Property    HostPort    : String  Read FHostPort     Write SetHostPort;
    Property    AutoStart   : Boolean Read FAutoStart    Write SetAutoStart;
    Property    AutoConnect : Boolean Read FAutoConnect  Write SetAutoConnect;
    Property    ActiveSlot  : Integer Read FActiveSlot   Write SetActiveSlot;
  Private
    Property    SlotSelection[ anIndex: Integer]: Boolean Read GetSlotSelection Write SetSlotSelection;
  End;

Var
  FormMain: TFormMain;

Implementation

{$R *.dfm}

// User area

Type

  TTestTracer = Class( TTracer)
  Private
    FStrings : TStrings;
  Public
    Constructor Create( aStrings: TStrings);
    Procedure   Trace( Const aMsg: String); Override;
  End;

    Constructor TTestTracer.Create( aStrings: TStrings);
    Begin
      Inherited Create;
      FStrings := aStrings;
      // If Assigned( FStrings)
      // Then FStrings.Clear;
    End;

    Procedure   TTestTracer.Trace( Const aMsg: String); // Override;
    Begin
      If Assigned( FStrings)
      Then FStrings.Add( aMsg);
    End;

//  Private

    Procedure   TFormMain.SetTestFileName( Const aValue: String);
    Begin
      If aValue <> FTestFileName
      Then Begin
        FTestFileName := aValue;
        EditTestFileName.Text := FTestFileName;
      End;
    End;

    Procedure   TFormMain.SetConnected( aValue: Boolean);
    Const
      Caps : Array[ Boolean] Of String = (
        '&Connect',
        'Dis&connect'
      );
      Connects : Array[ Boolean] Of String = (
        'Netmidi connection lost',
        'Netmidi connection made'
      );
    Begin
      If aValue <> FConnected
      Then Begin
        FConnected := aValue;
        BitBtnConnect.Caption := Caps[ FConnected];
        Log( Connects[ FConnected]);
      End;
    End;

    Procedure   TFormMain.SetRunning( avalue: Boolean);
    Const
      Caps : Array[ Boolean] Of String = (
        '&Start',
        '&Stop'
      );
    Begin
      If aValue <> FRunning
      Then Begin
        FRunning := aValue;
        BitBtnRunStop.Caption := Caps[ FRunning];
        TimerBeat    .Enabled := FRunning;
      End;
    End;

    Procedure   TFormMain.SetHostAddress( Const aValue: String);
    Begin
      If aValue <> FHostAddress
      Then Begin
        FHostAddress := aValue;
        EditHostAddress.Text := FHostAddress;
      End;
    End;

    Procedure   TFormMain.SetHostPort( Const aValue: String);
    Begin
      If aValue <> FHostPort
      Then Begin
        FHostPort := aValue;
        EditHostPort.Text := FHostPort;
      End;
    End;

    Procedure   TFormMain.SetAutoStart( aValue: Boolean);
    Begin
      If aValue <> FAutoStart
      Then Begin
        FAutoStart := aValue;
        CheckBoxAutoStart.Checked := FAutoStart;
      End;
    End;

    Procedure   TFormMain.SetAutoConnect( aValue: Boolean);
    Begin
      If aValue <> FAutoConnect
      Then Begin
        FAutoConnect := aValue;
        CheckBoxAutoConnect.Checked := FAutoConnect;
      End;
    End;

    Procedure   TFormMain.SetActiveSlot( aValue: Integer);
    Begin
      FActiveSlot := aValue;
    End;

    Function    TFormMain.GetSlotSelection( anIndex: Integer): Boolean;
    Begin
      Result := FSlotSelection[ anIndex];
    End;

    Procedure   TFormMain.SetSlotSelection( anIndex: Integer; aValue: Boolean);
    Begin
      FSlotSelection[ anIndex] := aValue;
      Case anIndex Of
        0 : SpeedButtonSlotA.Down := avalue;
        1 : SpeedButtonSlotB.Down := avalue;
        2 : SpeedButtonSlotC.Down := avalue;
        3 : SpeedButtonSlotD.Down := avalue;
      End;
    End;

//  Private

Const

  _d = 'Defaults';

    Procedure   TFormMain.LoadIni;
    Begin
      With TIniFile.Create( IniFileName) Do
        Try
          TestFileName := ReadString( _d, 'TestFileName', TestFileName);
          HostAddress  := ReadString( _d, 'HostAddress' , HostAddress );
          HostPort     := ReadString( _d, 'HostPort'    , HostPort    );
          AutoStart    := ReadBool  ( _d, 'AutoStart'   , AutoStart   );
          AutoConnect  := ReadBool  ( _d, 'AutoConnect' , AutoConnect );
        Finally
          Free;
        End;
    End;

    Procedure   TFormMain.SaveIni;
    Begin
      With TIniFile.Create( IniFileName) Do
        Try
          WriteString( _d, 'TestFileName', TestFileName);
          WriteString( _d, 'HostAddress' , HostAddress );
          WriteString( _d, 'HostPort'    , HostPort    );
          WriteBool  ( _d, 'AutoStart'   , AutoStart   );
          WriteBool  ( _d, 'AutoConnect' , AutoConnect );
        Finally
          Free;
        End;
    End;

//  Private

    Procedure   TFormMain.BrowseForTestFile;
    Begin
      With OpenDialogTest Do
      Begin
        If TestFileName <> ''
        Then FileName := TestFileName;
        If Execute
        Then TestFileName := FileName;
      End;
    End;

    Procedure   TFormMain.SendIAm;
    Var
      aMsg : TIamMessage;
    Begin
      aMsg := TIamMessage.Create;
      aMsg.SetVersion( 3, 3);
      FNmProtocolDriver.Send( aMsg);
    End;

    Procedure   TFormMain.SendActivateSlot( aTag: Integer;  aDown: Boolean);
    Var
      aMsg : TSlotActivatedMessage;
      bMsg : TSlotSelectedMessage;
      i    : Integer;
    Begin
      SlotSelection[ aTag] := aDown;
      aMsg := TSlotActivatedMessage.Create;
      If aDown
      Then ActiveSlot := aTag
      Else Begin
        For i := 0 To 3 Do
          If SlotSelection[ i]
          Then Begin
            ActiveSlot := i;
            Break;
          End;
      End;
      aMsg.ActiveSlot := ActiveSlot;
      FNmProtocolDriver.Send( aMsg);
      bMsg := TSlotSelectedMessage.Create;
      With bMsg Do
      Begin
        For i := 0 To 3 Do
          IsSelected[ i] := SlotSelection[ i];
      End;
      FNmProtocolDriver.Send( bMsg);
    End;


    Procedure   TFormMain.SendPatch;
    Var
      aPatchMessage: TPatchMessage;
    Begin
      TPatchMessage.UsePdlFile( ApplicationPath + 'patch.pdl');
      If Assigned( FPatch)
      Then FreeAndNil( FPatch);
      FPatch := TPatch.Create;
      FPatch.Loadpatch( TestFileName);
      aPatchMessage := TPatchMessage.Create( FPatch);
      aPatchMessage.Slot := ActiveSlot;
      Try
        FNmProtocolDriver.Send( aPatchMessage);
      Finally
        MemoResults.Update;
      End;
    End;

{$WARN UNSAFE_CODE OFF}

    Procedure   TFormMain.DoDriverData( aSender: TObject; aData: TByteList);
    Var
      aRawData : PBytes;
      aSize    : Integer;
      i        : Integer;
      S        : String;
    Begin
      If Connected
      Then
        With ClientSocket.Socket Do
        Begin
          aSize := aData.Size;
          GetMem( aRawData, aSize);
          Try
            S := '';
            For i := 0 To aSize - 1 Do
            Begin
              aRawData^[ i] := aData[ i];
              S := S + Format( ' %.2x', [ aData[ i]]);
            End;
            SendBuf( aRawData^, aSize);
            LogFmt( 'send:%s', [ S]);
          Finally
            FreeMem( aRawData, aSize);
          End;
        End
      Else Log( 'data not sent, no driver connected.');
    End;

{$WARN UNSAFE_CODE ON}

    Procedure   TFormMain.DoMidiMessage( aSender: TObject; aData: TMidiMessage);
    Begin
      If Assigned( aData)
      Then LogFmt( 'Received : %s', [ aData.ClassName])
      Else Log( 'Unparsable Midi mesage received');
    End;

    Procedure   TFormMain.Log( Const aMsg: String);
    Begin
      MemoResults.Lines.Add( aMsg);
    End;

    Procedure   TFormMain.LogFmt( Const aFormat: String; Const anArgs: Array Of Const);
    Begin
      Log( Format( aFormat, anArgs));
    End;

// Delpi area

procedure TFormMain.FormShow(Sender: TObject);
begin
  MemoResults.Clear;
  LoadIni;
  If AutoStart
  Then Running := True;
  If AutoConnect
  Then BitBtnConnectClick( Self);
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  TimerBeat.Enabled := False;
  FreeAndNil( FNmProtocolDriver);
  SaveIni;
end;

procedure TFormMain.BitBtnExitClick(Sender: TObject);
begin
  FStopping := True;
  ClientSocket.Close;
  Running := False;
  Close;
end;

procedure TFormMain.BitBtnBrowseTestClick(Sender: TObject);
begin
  BrowseForTestFile;
end;

procedure TFormMain.BitBtnBeamItClick(Sender: TObject);
begin
  SendPatch;
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  FNmProtocolDriver := TNMProtocolDriver.Create;
  With FNmProtocolDriver Do
  Begin
    OnDriverData  := DoDriverData;
    OnMidiMessage := DoMidiMessage;
  End;
  PanelTop.DoubleBuffered := True;
end;

procedure TFormMain.TimerBeatTimer(Sender: TObject);
begin
  FNmProtocolDriver.HeartBeat;
  FBeatState := ( FBeatState + 1) Mod 20;
  With ImageRun.Picture Do
  If FBeatState In [ 0, 1, 2, 9]
  Then LoadFromFile( ApplicationPath + 'Heart-1.bmp')
  Else LoadFromFile( ApplicationPath + 'Heart-2.bmp');
end;

procedure TFormMain.BitBtnRunStopClick(Sender: TObject);
begin
  Running := Not Running;
end;

procedure TFormMain.ClientSocketConnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  Connected := True;
end;

procedure TFormMain.ClientSocketDisconnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  Connected := False;
end;

procedure TFormMain.ClientSocketError(Sender: TObject;
  Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
begin
  With Socket Do
    LogFmt( '** Socket error %d', [ ErrorCode]);
  ErrorCode := 0;
end;

procedure TFormMain.ClientSocketLookup(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  Log( 'resolving host');
end;

{$WARN UNSAFE_CODE OFF}

procedure TFormMain.ClientSocketRead(Sender: TObject;
  Socket: TCustomWinSocket);
Var
  aData : PBytes;
  aSize : Integer;
begin
  If Not FStopping
  Then
    With Socket Do
    Begin
      aSize := ReceiveLength;
      While aSize > 0 Do
      Begin
        GetMem( aData, aSize);
        ReceiveBuf( aData^, aSize);
        Try
          FNmProtocolDriver.AcceptRawDriverData( aData, aSize);
        Finally
          FreeMem( aData, aSize);
        End;
        aSize := ReceiveLength;
      End;
  End;
end;

{$WARN UNSAFE_CODE ON}


procedure TFormMain.BitBtnConnectClick(Sender: TObject);
begin
  With ClientSocket Do
  Begin
    If Active
    Then Close
    Else Begin
      If IsDottedIp( HostAddress)
      Then Address := HostAddress
      Else Host    := HostAddress;
      Port := StrToInt( HostPort);
      Open;
    End;
  End;
end;

procedure TFormMain.EditHostAddressChange(Sender: TObject);
begin
  HostAddress := EditHostAddress.Text;
end;

procedure TFormMain.EditHostPortChange(Sender: TObject);
begin
  HostPort := EditHostPort.Text;
end;

procedure TFormMain.CheckBoxAutoStartClick(Sender: TObject);
begin
  AutoStart := CheckBoxAutoStart.Checked;
end;

procedure TFormMain.CheckBoxAutoConnectClick(Sender: TObject);
begin
  AutoConnect := CheckBoxAutoConnect.Checked;
end;

procedure TFormMain.SpeedButtonSlotClick(Sender: TObject);
begin
  With Sender As TSpeedButton Do
    SendActivateSlot( Tag, Down);
end;

procedure TFormMain.BitBtnClsClick(Sender: TObject);
begin
  MemoResults.Clear;
end;

procedure TFormMain.BitBtnSendIAmClick(Sender: TObject);
begin
  SendIAm;
end;

End.
