Unit FrmMain;

//  ////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2003 Jan Punter
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  ////////////////////////////////////////////////////////////////////////////
//
//    lupd : 2003-10-13
//
//  ////////////////////////////////////////////////////////////////////////////


Interface

Uses

  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, IniFiles,

  Globals, pdlparser, pdlStructs;

Type

  TFormMain = Class(TForm)
    OpenDialogPdl: TOpenDialog;
    PanelTop: TPanel;
    EditPdlFileName: TEdit;
    BitBtnBrowsePdl: TBitBtn;
    Label1: TLabel;
    BitBtnProcessPdl: TBitBtn;
    PanelMain: TPanel;
    BitBtnExit: TBitBtn;
    MemoResults: TMemo;
    CheckBoxPdlDebug: TCheckBox;
    procedure BitBtnBrowsePdlClick(Sender: TObject);
    procedure BitBtnProcessPdlClick(Sender: TObject);
    procedure BitBtnExitClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CheckBoxPdlDebugClick(Sender: TObject);
  Private
    FPdlFileName : String;
    FPdlDebug    : Boolean;
    FProtocol    : TProtocol;
  Private
    Procedure   SetPdlFileName( Const aValue: String);
    Procedure   SetPdlDebug( avalue: Boolean);
  Private
    Procedure   LoadIni;
    Procedure   SaveIni;
  Private
    Procedure   BrowseForPdlFile;
    Procedure   ProcessPdlFile;
  Private
    Property    PdlFileName: String  Read FPdlFileName Write SetPdlFileName;
    Property    PdlDebug   : Boolean Read FPdlDebug    Write SetPdlDebug;
  End;


Var

  FormMain: TFormMain;


Implementation


{$R *.dfm}

// User area

Type

  TTestTracer = Class( TTracer)
  Private
    FStrings : TStrings;
  Public
    Constructor Create( aStrings: TStrings);
    Procedure   Trace( Const aMsg: String); Override;
  End;

    Constructor TTestTracer.Create( aStrings: TStrings);
    Begin
      Inherited Create;
      FStrings := aStrings;
      If Assigned( FStrings)
      Then FStrings.Clear;
    End;

    Procedure   TTestTracer.Trace( Const aMsg: String); // Override;
    Begin
      If Assigned( FStrings)
      Then FStrings.Add( aMsg);
    End;

//  TFormMain.Private

    Procedure   TFormMain.SetPdlFileName( Const aValue: String);
    Begin
      If aValue <> FPdlFileName
      Then Begin
        FPdlFileName := aValue;
        EditPdlFileName.Text := FPdlFileName;
      End;
    End;

    Procedure   TFormMain.SetPdlDebug( avalue: Boolean);
    Begin
      If avalue <> FPdlDebug
      Then Begin
        FPdlDebug := aValue;
        CheckBoxPdlDebug.Checked := FPdlDebug;
      End;
    End;

//  TFormMain.Private

Const

  _d = 'Defaults';

    Procedure   TFormMain.LoadIni;
    Begin
      With TIniFile.Create( IniFileName) Do
        Try
          PdlFileName := ReadString( _d, 'PdlFileName', PdlFileName);
          PdlDebug    := ReadBool  ( _d, 'PdlDebug'   , PdlDebug    );
        Finally
          Free;
        End;
    End;

    Procedure   TFormMain.SaveIni;
    Begin
      With TIniFile.Create( IniFileName) Do
        Try
          WriteString( _d, 'PdlFileName', PdlFileName);
          WriteBool  ( _d, 'PdlDebug'   , PdlDebug    );
        Finally
          Free;
        End;
    End;

//  TFormMain.Private

    Procedure   TFormMain.BrowseForPdlFile;
    Begin
      With OpenDialogPdl Do
      Begin
        If PdlFileName <> ''
        Then FileName := PdlFileName;
        If Execute
        Then PdlFileName := FileName;
      End;
    End;

    Procedure   TFormMain.ProcessPdlFile;
    Var
      Stream : TBitStream;
      Data   : TIntStream;
      Packet : TPacket;
    Begin
      FProtocol := TProtocol.Create;
      With FProtocol Do
      Try
        UseTracer( TTestTracer.Create( MemoResults.Lines));
        TraceFmt( 'Load pdl file ''%s''', [ PdlFileName]);

        LoadPdlFile( PdlFileName, PdlDebug);

        Trace( 'pdl loaded and parsed.');

        Stream := TBitStream.Create;
        Packet := TPacket.Create;
        Try
          With Stream Do
          Begin
            Append( $f0, 8);

            Append( $33, 8);

            Append(   0, 1);
            Append( $16, 5);      // cc       =  22
            Append(   0, 2);      // slot     =   0

            Append( $06, 8);      // data

            Append(   0, 1);
            Append(  15, 7);      // pid1     =  15

            Append(   0, 1);
            Append( $7f, 7);

            Append(   0, 1);
            Append(  16, 7);      // pid2     =  16

            Append(   0, 1);
            Append( 127, 7);      // checksum = 127

            Append( $f7, 8);
          End;

          Trace( 'start parse stream');

          Packetparser[ 'Sysex'].Parse( Stream, Packet);
          Trace   ( 'stream parsed, trying to get data from the packet');
          TraceFmt( '  cc            = %d', [ Packet.Variable[ 'cc'  ]                      ]);
          TraceFmt( '  slot          = %d', [ Packet.Variable[ 'slot']                      ]);
          TraceFmt( '  data.name     = %s', [ Packet.Packet  [ 'data'].Name                 ]);
          TraceFmt( '  data.pid1     = %d', [ Packet.Packet  [ 'data'].Variable[ 'pid1'    ]]);
          TraceFmt( '  data.pid2     = %d', [ Packet.Packet  [ 'data'].Variable[ 'pid2'    ]]);
          TraceFmt( '  data.checksum = %d', [ Packet.Packet  [ 'data'].Variable[ 'checksum']]);

          Trace( 'stream parsed, trying to apply data now');

          Stream.Size := 0;
          Data := TIntStream.Create;
          With Data Do
          Begin
            Append(  22);  // cc
            Append(   0);  // slot
            Append(  16);  // pid1
            Append(  17);  // pid2
            Append( 127);  // checksum
          End;
          Packetparser[ 'Sysex'].Generate( Data, Stream);

          Trace( 'data applied to stream, now dump the stream');

          While Stream.HasAvailable( 8) Do
            TraceFmt( '  %.2x', [ Stream.GetInt( 8)]);

          Trace( 'stream dumped.');

        Finally
          Stream.Free;
        End;
        Trace( 'test done.');
      Finally
        FreeAndNil( FProtocol);
      End;
    End;


// Delphi area

procedure TFormMain.BitBtnBrowsePdlClick(Sender: TObject);
begin
  BrowseForPdlFile;
end;

procedure TFormMain.BitBtnProcessPdlClick(Sender: TObject);
begin
  ProcessPdlFile;
end;

procedure TFormMain.BitBtnExitClick(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  LoadIni;
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveIni;
end;

procedure TFormMain.CheckBoxPdlDebugClick(Sender: TObject);
begin
  PdlDebug := CheckBoxPdlDebug.Checked;
end;

End.

