Unit YaccLib;

// Yacc Library Unit for TP Yacc Version 3.0, 6-17-91 AG

// Modified to be used under Delphi 7 by Jan Punter. 2003-10-13.

Interface

Uses

	SysUtils, Globals, LexLib;

Const

  yymaxdepth = 8192; // default stack size of parser

Type

  YYSType = Integer; // default value type, may be redefined in Yacc output file

Var

  yychar   : Integer; // current lookahead character
  yynerrs  : Integer; // current number of syntax errors reported by the parser


  procedure yyerror( aMsg: ShortString);
  {
    error message printing routine used by the parser.
  }

  procedure yyclearin;
  {
    delete the current lookahead token.
  }

  procedure yyaccept;
  {
    trigger accept action of the parser; yyparse accepts returning 0, as if it
    reached end of input.
  }

  procedure yyabort;
  {
    like yyaccept, but causes parser to return with value 1, as if an
    unrecoverable syntax error had been encountered.
  }

  procedure yyerrlab;
  {
    causes error recovery to be started, as if a syntax error had been
    encountered.
  }

  procedure yyerrok;
  {
    when in error mode, resets the parser to its normal mode of operation.
  }



// Flags used internally by the parser routine:

Var

  yyflag : (
    yyfnone,
    yyfaccept,
    yyfabort,
    yyferror
  );
  yyerrflag : Integer;



Implementation



  Procedure yyerror( aMsg: ShortString);
	Var
		s: Integer;
    e: Integer;
		t: string;
	Begin
    Inc( yyErrorCount);
    yyOutput.
      Add(
        Format(
          '[%d,%d] in file "%s": "%s"',
          [
            yylineno,
            yycolno,
            ExtractFileName( yyFileName),
            aMsg
          ]
        )
      );
		// yyoutput.Add( msg + ': at character # ' + IntToStr( yycolno));

		s := yyposition;
		While ( s > 1) And ( yyinput_text[ s - 1] <> #10) Do
      Dec( s);
		e := yyposition;
		While ( e < Length( yyinput_text)) And ( yyinput_text[ e + 1] <> #13) Do
      Inc( e);
		t := Copy( yyinput_text, s, e - s + 1);
		yyoutput.Add( t);
    If yyposition > s
    Then FillChar( t[ 1], yyposition - s, ' ');
		SetLength( t , yyposition - s + 1);
    t[ yyposition - s + 1] := '^';
		yyoutput.Add( t);
	End;


  Procedure yyclearin;
  Begin
    yychar := -1;
	End;


  Procedure yyaccept;
  begin
    yyflag := yyfaccept;
  End;


  Procedure yyabort;
  Begin
    yyflag := yyfabort;
  End;


  Procedure yyerrlab;
  Begin
    yyflag := yyferror;
  End;

  
  Procedure yyerrok;
  Begin
    yyerrflag := 0;
  End;

End.

