unit FrmMain;

{

    NetMidi server - a tool to stream MIDI data over TCP/IP.

    Copyright (C) 2003  Blue Hell / Jan Punter :: J.Punter@iaf.nl

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Modification history :

    2003-10-16, created.
    2003-10-23, bugfix - tcp data sent out was corrupted.

}

Interface

// get rid of .net related warnings.

{$WARN UNSAFE_CODE OFF}
{$WARN UNSAFE_TYPE OFF}

Uses

  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, mmSystem, MidiType, MidiOut, MidiIn, ScktComp, ExtCtrls,
  Buttons, WinSock, TrayIcon, IniFiles, Globals, FrmAbout, FrmTest;

Type

  TFormMain = Class(TForm)
    PanelBottom: TPanel;
    PanelMain: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    ComboBoxInputs: TComboBox;
    ComboBoxOutputs: TComboBox;
    ServerSocket: TServerSocket;
    MidiInput: TMidiInput;
    MidiOutput: TMidiOutput;
    TimerMidiRx: TTimer;
    TimerMidiTx: TTimer;
    Label3: TLabel;
    EditTcpPort: TEdit;
    CheckBoxDisableNagle: TCheckBox;
    PanelTop: TPanel;
    BitBtnStart: TBitBtn;
    BitBtnExit: TBitBtn;
    TrayIcon: TTrayIcon;
    PanelConnections: TPanel;
    ListBoxConnections: TListBox;
    PanelActivity: TPanel;
    SpeedButtonDisconnect: TSpeedButton;
    TimerTcpRx: TTimer;
    TimerTcpTx: TTimer;
    PanelTcpRx: TPanel;
    PanelTcpTx: TPanel;
    PanelMidiRx: TPanel;
    PanelMidiTx: TPanel;
    Label4: TLabel;
    Label5: TLabel;
    CheckBoxAutoStart: TCheckBox;
    BitBtnAbout: TBitBtn;
    BitBtnTest: TBitBtn;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BitBtnExitClick(Sender: TObject);
    procedure TrayIconClick(Sender: TObject);
    procedure FormCanResize(Sender: TObject; var NewWidth,
      NewHeight: Integer; var Resize: Boolean);
    procedure TimerMidiRxTimer(Sender: TObject);
    procedure TimerMidiTxTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MidiInputMidiInput(Sender: TObject);
    procedure SpeedButtonDisconnectClick(Sender: TObject);
    procedure BitBtnStartClick(Sender: TObject);
    procedure ServerSocketClientConnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ServerSocketClientDisconnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ServerSocketClientRead(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ServerSocketListen(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure CheckBoxDisableNagleClick(Sender: TObject);
    procedure ComboBoxInputsChange(Sender: TObject);
    procedure ComboBoxOutputsChange(Sender: TObject);
    procedure CheckBoxAutoStartClick(Sender: TObject);
    procedure TimerTcpRxTimer(Sender: TObject);
    procedure TimerTcpTxTimer(Sender: TObject);
    procedure BitBtnAboutClick(Sender: TObject);
    procedure BitBtnTestClick(Sender: TObject);
  Private
      FInitialized        : Boolean;
      FInSelection        : String;
      FOutSelection       : String;
  Private
      FMidiInName         : String;
      FMidiOutName        : String;
      FMidiInOpen         : Boolean;
      FMidiOutOpen        : Boolean;
      FPort               : Integer;
      FIsConnected        : Boolean;
      FServerName         : String;
      FOrigCaption        : String;
      FConnections        : TStringList;
      FLockConnectUpdates : Integer;
      FAutoStart          : Boolean;
      FDisableNagle       : Boolean;
  Private
    Procedure   Initialize;
    Procedure   SetInSelection ( Const aValue: String);
    Procedure   SetOutSelection( Const aValue: String);
  Private
    Procedure   SetMidiInName  ( Const aValue: String);
    Procedure   SetMidiOutName ( Const aValue: String);
    Procedure   SetMidiInOpen  ( aValue: Boolean);
    Procedure   SetMidiOutOpen ( aValue: Boolean);
    Procedure   SetIsConnected ( aValue: Boolean);
    Procedure   SetAutoStart   ( aValue: Boolean);
    Procedure   SetDisableNagle( aValue: Boolean);
  Private
    Procedure   UpdateCaption;
    Procedure   LockConnectUpdates;
    Procedure   UnlockConnectUpdates;
  Private
    Procedure   SelectMidiDevices;
    Procedure   DoConnect( aValue: Boolean);
    Procedure   SetNagleOption( aSock: TSocket);
    Function    FormatConnectInfo( aSocket: TCustomWinSocket): String;
    Procedure   AddConnection( aSocket: TCustomWinSocket);
    Procedure   RemConnection( aSocket: TCustomWinSocket);
    Procedure   UpdateConnections;
    Procedure   DisConnectClient( Const aConnectInfo: String);
  Private
    Procedure   SaveIni;
    Procedure   LoadIni;
    Procedure   SignalMidiRx;
    Procedure   SignalMidiTx;
    Procedure   SignalTcpRx;
    Procedure   SignalTcpTx;
    Procedure   AcceptMidiData( Const aData: Array Of Byte);
    Procedure   SendMidiData  ( Const aData: Array Of Byte);
    Procedure   AcceptTcpData ( Const aData: Array Of Byte);
    Procedure   SendTcpData   ( Const aData: Array Of Byte);
  Public
    Property    InSelection  : String   Read FInSelection      Write SetInSelection;
    Property    OutSelection : String   Read FOutSelection     Write SetOutSelection;
  Public
    Property    MidiInName       : String   Read FMidiInName       Write SetMidiInName;
    Property    MidiOutName      : String   Read FMidiOutName      Write SetMidiOutName;
    Property    MidiInOpen       : Boolean  Read FMidiInOpen       Write SetMidiInOpen;
    Property    MidiOutOpen      : Boolean  Read FMidiOutOpen      Write SetMidiOutOpen;
    Property    Port             : Integer  Read FPort             Write FPort;
    Property    IsConnected      : Boolean  Read FIsConnected      Write SetIsConnected;
    Property    ServerName       : String   Read FServerName       Write FServerName;
    Property    AutoStart        : Boolean  Read FAutoStart        Write SetAutoStart;
    Property    DisableNagle     : Boolean  Read FDisableNagle     Write SetDisableNagle;
  end;

Var

  FormMain: TFormMain;

Implementation

{$R *.dfm}

// User area

Const

  CNoteOff         = $8; // Length = 3;
  CNoteOn          = $9; // Length = 3;
  CKeyPressure     = $a; // Length = 3;
  CControlChange   = $b; // Length = 3;
  CProgramChange   = $c; // Length = 3;
  CChannelPressure = $d; // Length = 2;
  CPitchBend       = $e; // Length = 2;
  CSysEx           = $f; // Length = 3;

  // When ( aByte Shr 4) And $0f = CSysEx
  // Low nibble of 1st byte

  SSysEx           = $0; // Length = n;
  SSongPosition    = $2; // Length = 3;
  SSongSelect      = $3; // Length = 2;
  STuneRequest     = $6; // Length = 1;
  SEox             = $7; // Length = 1;
  STimingClock     = $8; // Length = 1;
  SStart           = $a; // Length = 1;
  SContinue        = $b; // Length = 1;
  SStop            = $c; // Length = 1;
  SActiveSense     = $e; // Length = 1;
  SReset           = $f; // Length = 1;




    Procedure   TFormMain.Initialize;
    Var
      i        : Integer;
      aInCaps  : TMidiInCaps;
      aOutCaps : TMidiOutCaps;
      nDevs    : Integer;
    Begin
      If Not FInitialized
      Then Begin
        // Inputs
        With ComboBoxInputs Do
        Begin
          Clear;
          FInSelection := '';
          nDevs := MidiInGetNumDevs;
          If nDevs > 0
          Then
            For i := 0 To nDevs - 1 Do
            Begin
              MidiInGetDevCaps( i, @ aInCaps, SizeOf( aInCaps));
              Items.Add( StrPas( aInCaps.szPName));
            End;
          If Items.Count > 0
          Then InSelection := Items[ 0];
        End;
        // Outputs
        With ComboBoxOutputs Do
        Begin
          Clear;
          FOutSelection := '';
          MidiOutGetDevCaps( MIDI_MAPPER, @ aOutCaps, SizeOf( aOutCaps));
          Items.Add( StrPas( aOutCaps.szPName));
          nDevs := MidiOutGetNumDevs;
          If nDevs > 0
          Then
            For i := 0 To nDevs - 1 Do
            Begin
              MidiOutGetDevCaps( i, @ aOutCaps, SizeOf( aOutCaps));
              Items.Add( StrPas( aOutCaps.szPName));
            End;
          If Items.Count > 0
          Then OutSelection := Items[ 0];
        End;
        // Flag initialization
        FInitialized := True;
      End;
    End;

    Procedure   TFormMain.SetInSelection( Const aValue: String);
    Var
      i : Integer;
    Begin
      With ComboBoxInputs Do
      Begin
        ItemIndex := -1;
        For i := 0 To Items.Count - 1 Do
          If UpperCase( aValue) = UpperCase( Items[ i])
          Then Begin
            ItemIndex := i;
            FInSelection := Items[ i];
            Break;
          End;
      End;
    End;

    Procedure   TFormMain.SetOutSelection( Const aValue: String);
    Var
      i : Integer;
    Begin
      With ComboBoxOutputs Do
      Begin
        ItemIndex := -1;
        For i := 0 To Items.Count - 1 Do
          If UpperCase( aValue) = UpperCase( Items[ i])
          Then Begin
            ItemIndex := i;
            FOutSelection := Items[ i];
            Break;
          End;
      End;
    End;

//  Private

    Procedure   TFormMain.SetMidiInName( Const aValue: String);
    Var
      WasOpen : Boolean;
    Begin
      WasOpen := MidiInOpen;
      If WasOpen
      Then MidiInOpen := False;
      Try
        If aValue <> ''
        Then Begin
          With MidiInput Do
            ProductName := aValue;
          If WasOpen
          Then MidiInOpen := True;
        End;
        FMidiInName := aValue;
      Except
        On E:Exception Do
          ShowMessageFmt( 'Could not open midi in port : %s', [ E.Message]);
      End;
    End;

    Procedure   TFormMain.SetMidiOutName( Const aValue: String);
    Var
      WasOpen : Boolean;
    Begin
      WasOpen := MidiOutOpen;
      If WasOpen
      Then MidiOutOpen := False;
      Try
        If aValue <> ''
        Then Begin
          With MidiOutput Do
            ProductName := aValue;
          If WasOpen
          Then MidiOutOpen := True;
        End;
        FMidiOutName := aValue;
      Except
        On E:Exception Do
          ShowMessageFmt( 'Could not open midi out port : %s', [ E.Message]);
      End;
    End;

    Procedure   TFormMain.SetMidiInOpen( aValue: Boolean);
    Begin
      If aValue <> FMidiInOpen
      Then Begin
        If aValue
        Then
          With MidiInput Do
          Begin
            Open;
            Start;
            FMidiInOpen := True;
          End
        Else
          With MidiInput Do
          Begin
            Try
              Stop;
            Finally
              Close;
              FMidiInOpen := False;
            End;
          End;
      End;
    End;

    Procedure   TFormMain.SetMidiOutOpen( aValue: Boolean);
    Begin
      If aValue <> FMidiOutOpen
      Then Begin
        If aValue
        Then MidiOutput.Open
        Else MidiOutput.Close;
        FMidiOutOpen := aValue;
      End;
    End;

    Procedure   TFormMain.SetIsConnected( aValue: Boolean);
    Begin
      If FIsConnected <> aValue
      Then Begin
        FIsConnected := aValue;
        UpdateCaption;
        PanelMain.Enabled := Not IsConnected;
      End;
    End;

    Procedure   TFormMain.SetAutoStart( aValue: Boolean);
    Begin
      If aValue <> FAutoStart
      Then Begin
        FAutoStart := aValue;
        CheckBoxAutoStart.Checked := FAutoStart;
      End;
    End;

    Procedure   TFormMain.SetDisableNagle( aValue: Boolean);
    Begin
      If aValue <> FDisableNagle
      Then Begin
        FDisableNagle := aValue;
        CheckBoxDisableNagle.Checked := FDisableNagle;
      End;
    End;

//  Private

    Procedure   TFormMain.UpdateCaption;
    Const
      Connects : Array[ Boolean] Of String = (
        'not listening',
        'listening'
      );
      Buttons : Array[ Boolean] Of String = (
        '&Start',
        '&Stop'
      );
    Var
      i : Integer;
    Begin
      Caption := Format( '%s - (%s)', [ FOrigCaption, Connects[ IsConnected]]);
      With PanelMain Do
      Begin
        For i := 0 To ControlCount - 1 Do
          Controls[ i].Enabled := Not IsConnected;
      End;
      BitBtnStart.Caption := Buttons[ IsConnected];
    End;

    Procedure   TFormMain.LockConnectUpdates;
    Begin
      Inc( FLockConnectUpdates);
    End;

    Procedure   TFormMain.UnlockConnectUpdates;
    Begin
      If FLockConnectUpdates > 0
      Then Dec( FLockConnectUpdates);
      If FLockConnectUpdates = 0
      Then UpdateConnections;
    End;

//  Private

    Procedure   TFormMain.SelectMidiDevices;
    Begin
      Try
        MidiInName := InSelection;
        If Not MidiInOpen
        Then MidiInOpen := MidiInName <> '';
      Except
        On Exception Do
          ;
      End;
      Try
        MidiOutName := OutSelection;
        If Not MidiOutOpen
        Then MidiOutOpen := MidiOutName <> '';
      Except
        On Exception Do
          ;
      End;
    End;

    Procedure   TFormMain.DoConnect( aValue: Boolean);
    Begin
      If aValue <> IsConnected
      Then Begin
        If aValue
        Then Begin
          SelectMidiDevices;
          Port := StrToInt( EditTcpPort.Text);
          ServerSocket.Port := Port;
          ServerSocket.Open;
        End
        Else Begin
          MidiInput.Stop;
          MidiInput.Close;
          MidiOutput.Close;
          ServerSocket.Close;
          IsConnected := False;
          FConnections.Clear;
          UpdateConnections;
        End;
      End;
    End;

    Procedure   TFormMain.SetNagleOption( aSock: TSocket);
    Var
      SockRes : Integer;
      NoDelay : LongBool;
    Begin
      NoDelay  := DisableNagle;
      SockRes  := setsockopt( aSock, IPPROTO_TCP, TCP_NODELAY, @ NoDelay, SizeOf( NoDelay));
      If SockRes <> 0
      Then Begin
        SockRes := WSAGetLastError;
        Raise Exception.CreateFmt( 'Error (%d) setting Nagling to %s', [ SockRes, BoolToStr( Not DisableNagle)]);
      End;
    End;

    Function    TFormMain.FormatConnectInfo( aSocket: TCustomWinSocket): String;
    Begin
      With aSocket Do
        Result := Format( '%s, %s : %d', [ RemoteHost, RemoteAddress, RemotePort]);
    End;

    Procedure   TFormMain.AddConnection( aSocket: TCustomWinSocket);
    Begin
      FConnections.Add( FormatConnectInfo( aSocket));
      UpdateConnections;
    End;

    Procedure   TFormMain.RemConnection( aSocket: TCustomWinSocket);
    Var
      i : Integer;
    Begin
      With FConnections Do
      Begin
        i := IndexOf( FormatConnectInfo( aSocket));
        If i >=0
        Then Delete( i);
      End;
      UpdateConnections;
    End;

    Procedure   TFormMain.UpdateConnections;
    Var
      i : Integer;
    Begin
      If FLockConnectUpdates = 0
      Then
        With ListBoxConnections Do
        Begin
          Items.Clear;
          With FConnections Do
            For i := 0 To Count - 1 Do
              Items.Add( Strings[ i]);
        End;
    End;

    Procedure   TFormMain.DisConnectClient( Const aConnectInfo: String);
    Var
      i : Integer;
    Begin
      With ServerSocket.Socket Do
      Begin
        For i := 0 To ActiveConnections - 1 Do
          If FormatConnectInfo( Connections[ i]) = aConnectInfo
          Then Connections[ i].Close;
      End;
    End;

//  Private

    Procedure   TFormMain.SaveIni;
    Begin
      With TIniFile.Create( IniFileName) Do
        Try
          WriteInteger( 'Settings', 'Left'        , Left        );
          WriteInteger( 'Settings', 'Top'         , Top         );
          WriteInteger( 'Settings', 'Width'       , Width       );
          WriteInteger( 'Settings', 'Height'      , Height      );
          WriteString ( 'Settings', 'MidiIn'      , FMidiInName );
          WriteString ( 'Settings', 'MidiOut'     , FMidiOutName);
          WriteInteger( 'Settings', 'TCPIP'       , Port        );
          WriteBool   ( 'Settings', 'AutoStart'   , AutoStart   );
          WriteBool   ( 'Settings', 'DisableNagle', DisableNagle);
        Finally
          Free;
        End;
    End;

    Procedure   TFormMain.LoadIni;
    Var
      L, T, W, H : Integer;
    Begin
      With TIniFile.Create( IniFileName) Do
        Try
          L            := ReadInteger( 'Settings', 'Left'        , Left         );
          T            := ReadInteger( 'Settings', 'Top'         , Top          );
          W            := ReadInteger( 'Settings', 'Width'       , Width        );
          H            := ReadInteger( 'Settings', 'Height'      , Height       );
          InSelection  := ReadString ( 'Settings', 'MidiIn'      , FInSelection );
          OutSelection := ReadString ( 'Settings', 'MidiOut'     , FOutSelection);
          Port         := ReadInteger( 'Settings', 'TCPIP'       , Port         );
          AutoStart    := ReadBool   ( 'Settings', 'AutoStart'   , AutoStart    );
          DisableNagle := ReadBool   ( 'Settings', 'DisableNagle', DisableNagle );

          With Screen Do
          Begin
            If ( L < - 8) Or ( L > Width - 50)
            Then L := 0;
            If ( T < - 8) Or ( T > Height - 50)
            Then T := 0;
            If ( W < 20) Or ( W > Width + 16)
            Then W := 20;
            If ( H < 10) Or ( H > Height + 16)
            Then H := 10;
          End;
          Left   := L;
          Top    := T;
          Width  := W;
          Height := H;
        Finally
          Free;
        End;
    End;

    Procedure   TFormMain.SignalMidiRx;
    Begin
      With TimerMidiRx Do
      Begin
        Enabled := False;
        Enabled := True;
      End;
      PanelMidiRx.Color := clYellow;
    End;

    Procedure   TFormMain.SignalMidiTx;
    Begin
      With TimerMidiTx Do
      Begin
        Enabled := False;
        Enabled := True;
      End;
      PanelMidiTx.Color := clYellow;
    End;

    Procedure   TFormMain.SignalTcpRx;
    Begin
      With TimerTcpRx Do
      Begin
        Enabled := False;
        Enabled := True;
      End;
      PanelTcpRx.Color := clYellow;
    End;

    Procedure   TFormMain.SignalTcpTx;
    Begin
      With TimerTcpTx Do
      Begin
        Enabled := False;
        Enabled := True;
      End;
      PanelTcpTx.Color := clYellow;
    End;

    Procedure   TFormMain.AcceptMidiData( Const aData: Array Of Byte);
    Begin
      SignalMidiRx;
      SendTcpData( aData);
    End;

    Procedure   TFormMain.SendMidiData( Const aData: Array Of Byte);
    Var
      aMidiEvent : TMyMidiEvent;
      aSize      : Integer;
      i          : Integer;
    Begin
      If MidiOutOpen
      Then Begin
        aMidiEvent := TMyMidiEvent.Create;
        aSize := Length( aData);
        aMidiEvent.SysexLength := aSize;
        GetMem( aMidiEvent.Sysex, aSize);
        For i := 0 To aSize - 1 Do
          aMidiEvent.Sysex[ i] := Char( aData[ i]);
        Try
          MidiOutput.PutMidiEvent( aMidiEvent);
          SignalMidiTx;
        Except
          On Exception Do
            ;
        End;
        aMidiEvent.Free;
      End;
    End;

    Procedure   TFormMain.AcceptTcpData( Const aData: Array Of Byte);
    Begin
      SendMidiData( aData);
    End;

    Procedure   TFormMain.SendTcpData( Const aData: Array Of Byte);
    Var
      s       : Integer;
      i       : Integer;
      aBuffer : PBytes;
      aSize   : Integer;
    Begin
      // Send data out over tcp/ip.
      Try
        aSize := Length( aData);
        If IsConnected And ( aSize > 0)
        Then
          With ServerSocket.Socket Do
            For s := 0 To ActiveConnections - 1 Do
              With Connections[ s] Do
                If Connected
                Then
                  Try
                    GetMem( aBuffer, aSize);
                    For i := 0 To aSize - 1 Do
                      aBuffer[ i] := aData[ i];
                    SendBuf( aBuffer^, aSize);
                    SignalTcpTx;
                  Finally
                    FreeMem( aBuffer, aSize);
                  End;
      Except
        On Exception Do
          ;
      End;
    End;


// Delphi area



procedure TFormMain.FormShow(Sender: TObject);
begin
  If AutoStart And Not IsConnected
  Then DoConnect( True);
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveIni;
end;

procedure TFormMain.BitBtnExitClick(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.TrayIconClick(Sender: TObject);
begin
  TrayIcon.Restore;
end;

procedure TFormMain.FormCanResize(Sender: TObject; var NewWidth,
  NewHeight: Integer; var Resize: Boolean);
begin
  Resize := False;
end;

procedure TFormMain.TimerMidiRxTimer(Sender: TObject);
begin
  With TimerMidiRx Do
    Enabled := False;
  PanelMidiRx.Color := clBtnFace;
end;

procedure TFormMain.TimerMidiTxTimer(Sender: TObject);
begin
  With TimerMidiTx Do
    Enabled := False;
  PanelMidiTx.Color := clBtnFace;
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  FOrigCaption := Caption;
  UpdateCaption;

  FConnections := TStringList.Create;
  With FConnections Do
  Begin
    Sorted     := True;
    Duplicates := dupIgnore;
  End;

  FMidiInName   := '';
  FMidiOutname  := '';
  FInSelection  := '';
  FOutSelection := '';
  FIsConnected  := True;
  IsConnected   := False;
  DisableNagle  := True;  // TCP Nagling off by default - better real time behaviour, maybe ...

  Initialize;
  LoadIni; // Reads FPreMidiXxx variables.
  SelectMidiDevices;
end;

procedure TFormMain.MidiInputMidiInput(Sender: TObject);
Type

  MidiMsg = packed Record
    MSg : Byte;
    d1  : Byte;
    d2  : Byte;
  End;

  Function CombineBytes( aMsg, ad1, ad2: Byte): MidiMsg;
  Begin
    With Result Do
    Begin
      Msg := aMsg;
      d1  := ad1;
      d2  := ad2;
    End;
  End;

Var
  i          : Integer;
  aMidiEvent : TMyMidiEvent;
  aSize      : Byte;
  aData      : Array Of Byte;
begin
  With MidiInput Do
  Begin
    aMidiEvent := GetMidiEvent;
    Try
      With aMidiEvent Do
      Begin
        If SysExLength > 0
        Then Begin
          SetLength( aData, SysExLength);
          For i := 0 To SysExLength - 1 Do
            aData[ i] := Byte( SysEx[ i]);
          AcceptMidiData( aData);
        End
        Else Begin
          Case ( MidiMessage And $f0) Shr 4 Of
            CProgramChange,
            CChannelPressure : aSize := 2;
            CSysEx : Begin
              Case MidiMessage And $0f Of
                SSongPosition : aSize := 3;
                SSongSelect   : aSize := 2;
                STuneRequest,
                SEox,
                STimingClock,
                SStart,
                SContinue,
                SStop,
                SActiveSense,
                SReset : aSize := 1;
                Else aSize := 0;
              End;
            End;
            Else aSize := 3;
          End;
          If aSize > 0
          Then Begin
            SetLength( aData, aSize);
            aData[ 0] := MidiMessage;
            If aSize > 1
            Then aData[ 1] := Data1;
            If aSize > 2
            Then aData[ 2] := Data2;
            AcceptMidiData( aData);
          End;
        End;
      End;
    Finally
      aMidiEvent.Free;
    End;
  End;
end;

procedure TFormMain.SpeedButtonDisconnectClick(Sender: TObject);
Var
  i : Integer;
begin
  With ListBoxConnections Do
    With Items Do
    Begin
      LockConnectUpdates;
      Try
        For i := 0 To Count - 1 Do
          If Selected[ i]
          Then DisconnectClient( Items[ i])
      Finally
        UnlockConnectUpdates;
      End;
    End;
end;

procedure TFormMain.BitBtnStartClick(Sender: TObject);
begin
  DoConnect( Not IsConnected);
end;

procedure TFormMain.ServerSocketClientConnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  SetNagleOption( Socket.SocketHandle);
  AddConnection( Socket);
end;

procedure TFormMain.ServerSocketClientDisconnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  RemConnection( Socket);
end;

procedure TFormMain.ServerSocketClientRead(Sender: TObject;
  Socket: TCustomWinSocket);
Var
  aData   : Array Of Byte;
  aSize   : Integer;
  aBuffer : PBytes;
  i       : Integer;
begin
  SignalTcpRx;
  With Socket Do
  Begin
    aSize := ReceiveLength;
    While aSize > 0 Do
    Begin
      GetMem( aBuffer, aSize);
      Try
        ReceiveBuf( aBuffer^, aSize);
        SetLength( aData, aSize);
        For i := 0 To aSize - 1 Do
          aData[ i] := aBuffer[ i];
        AcceptTcpData( aData);
      Finally
        FreeMem( aBuffer, aSize);
      End;
      aSize := ReceiveLength;
    End;
  End;
end;

procedure TFormMain.ServerSocketListen(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  IsConnected := True
end;

procedure TFormMain.CheckBoxDisableNagleClick(Sender: TObject);
begin
  DisableNagle := CheckBoxDisableNagle.Checked;
end;

procedure TFormMain.ComboBoxInputsChange(Sender: TObject);
begin
  With ComboBoxInputs Do
    If ItemIndex >= 0
    Then InSelection := Items[ ItemIndex];
end;

procedure TFormMain.ComboBoxOutputsChange(Sender: TObject);
begin
  With ComboBoxOutputs Do
    If ItemIndex >= 0
    Then OutSelection := Items[ ItemIndex];
end;

procedure TFormMain.CheckBoxAutoStartClick(Sender: TObject);
begin
  AutoStart := CheckBoxAutoStart.Checked;
end;

procedure TFormMain.TimerTcpRxTimer(Sender: TObject);
begin
  With TimerTcpRx Do
    Enabled := False;
  PanelTcpRx.Color := clBtnFace;
end;

procedure TFormMain.TimerTcpTxTimer(Sender: TObject);
begin
  With TimerTcpTx Do
    Enabled := False;
  PanelTcpTx.Color := clBtnFace;
end;

procedure TFormMain.BitBtnAboutClick(Sender: TObject);
begin
  FormAbout.ShowModal;
end;

procedure TFormMain.BitBtnTestClick(Sender: TObject);
begin
  FormTest.Execute( FPort);
end;

End.
