Unit FrmMain;

//  ////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2003 Jan Punter
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  ////////////////////////////////////////////////////////////////////////////
//
//    lupd : 2003-11-23
//
//  ////////////////////////////////////////////////////////////////////////////


Interface

Uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, Inifiles, ScktComp,

  Globals, pdlstructs, patchstructs, protocolstructs;

Const

  CommandBufSize = 65535;                             // Commandline size
  SpecialSymbol  = 'patchloader_commandLine_message'; // Unique message Id

Type

  TFormMain = Class(TForm)
    PanelTop: TPanel;
    Label1: TLabel;
    EditTestFileName: TEdit;
    BitBtnBrowseTest: TBitBtn;
    BitBtnBeamIt: TBitBtn;
    BitBtnExit: TBitBtn;
    PanelMain: TPanel;
    MemoResults: TMemo;
    OpenDialogTest: TOpenDialog;
    ClientSocket: TClientSocket;
    TimerBeat: TTimer;
    BitBtnRunStop: TBitBtn;
    EditHostAddress: TEdit;
    EditHostPort: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    BitBtnConnect: TBitBtn;
    ImageRun: TImage;
    CheckBoxAutoStart: TCheckBox;
    CheckBoxAutoConnect: TCheckBox;
    BitBtnCls: TBitBtn;
    TimerCommand: TTimer;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BitBtnExitClick(Sender: TObject);
    procedure BitBtnBrowseTestClick(Sender: TObject);
    procedure BitBtnBeamItClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TimerBeatTimer(Sender: TObject);
    procedure BitBtnRunStopClick(Sender: TObject);
    procedure ClientSocketConnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientSocketDisconnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientSocketError(Sender: TObject; Socket: TCustomWinSocket;
      ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure ClientSocketLookup(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientSocketRead(Sender: TObject; Socket: TCustomWinSocket);
    procedure BitBtnConnectClick(Sender: TObject);
    procedure EditHostAddressChange(Sender: TObject);
    procedure EditHostPortChange(Sender: TObject);
    procedure CheckBoxAutoStartClick(Sender: TObject);
    procedure CheckBoxAutoConnectClick(Sender: TObject);
    procedure BitBtnClsClick(Sender: TObject);
    procedure TimerCommandTimer(Sender: TObject);
  Private
    FTestFileName     : String;
    FCommands         : TStringList;
    FNmProtocolDriver : TNMProtocolDriver;
    FPatches          : Array[ 0 .. 3] Of TPatch;
    FSlotSelection    : Array[ 0 .. 3] Of Boolean;
    FLastSlot         : Integer;
    FBeatState        : Integer;
    FConnected        : Boolean;
    FRunning          : Boolean;
    FHostAddress      : String;
    FHostPort         : String;
    FAutoStart        : Boolean;
    FAutoConnect      : Boolean;
    FStopping         : Boolean;
  Private
    Procedure   SetTestFileName( Const aValue: String);
    Procedure   SetConnected( aValue: Boolean);
    Procedure   SetRunning  ( avalue: Boolean);
    Procedure   SetHostAddress( Const aValue: String);
    Procedure   SetHostPort   ( Const aValue: String);
    Procedure   SetAutoStart  ( aValue: Boolean);
    Procedure   SetAutoConnect( aValue: Boolean);
  Private
    Procedure   LoadIni;
    Procedure   SaveIni;
  Private
    Procedure   Log( Const aMsg: String);
    Procedure   LogFmt( Const aFormat: String; Const anArgs: Array Of Const);
    Procedure   BrowseForTestFile;
    Procedure   DeselectSlots;
    Procedure   SelectSlot( aSlot: Integer);
    Procedure   SendSlotSelection;
    Procedure   SetActiveSlot( aSlot: Integer);
    Procedure   SendPatch( aPatch: TPatch; aSlot: Integer);
    Procedure   SendPatchFile( Const aPatchFileName: String; aSlot: Integer);
    Procedure   DoDriverData  ( aSender: TObject; aData: TByteList   );
    Procedure   DoMidiMessage ( aSender: TObject; aData: TMidiMessage);
    Procedure   DoOnUnparsable( aSender: TObject; aData: TByteList   );
    Procedure   AcceptCommandLine( aCommandLine: Atom);         Overload;
    Procedure   AcceptCommandLine( Const aCommandLine: String); Overload;
    Procedure   ProcessCommands;
    Procedure   ProcessCommand( Const aCommand: String);               Overload;
    Procedure   ProcessCommand( Const aCommand, anArg: String);        Overload;
  Private
    Property    TestFileName: String  Read FTestFileName Write SetTestFileName;
    Property    Connected   : Boolean Read FConnected    Write SetConnected;
    Property    Running     : Boolean Read FRunning      Write SetRunning;
    Property    HostAddress : String  Read FHostAddress  Write SetHostAddress;
    Property    HostPort    : String  Read FHostPort     Write SetHostPort;
    Property    AutoStart   : Boolean Read FAutoStart    Write SetAutoStart;
    Property    AutoConnect : Boolean Read FAutoConnect  Write SetAutoConnect;
  End;

Var
  FormMain: TFormMain;

Implementation

{$R *.dfm}

// User area

// Some special command line stuff used by a second invocation.
//

Var

{$WARN UNSAFE_TYPE OFF}

  OldWindowProc : Pointer; // Variable for the old windows proc
  CmdLineMsg    : DWord;   // Custum systemwide message

  Function NewWindowProc(
    aWindowHandle: hWnd;
    aMessage     : Cardinal;
    aParamW      : LongInt;
    aPAramL      : LongInt
  ): LongInt stdCall;
  Begin
    If aMessage = CmdLineMsg
    Then Begin
      // SendMessage( Application.Handle, WM_SYSCOMMAND, SC_RESTORE, 0);
      // SetForeGroundWindow( Application.Handle);
      FormMain.AcceptCommandLine( Atom( aParamW));
      Result := 0;
      Exit;
    End;
    Result :=
      CallWindowProc(
        OldWindowProc,
        aWindowHandle,
        aMessage,
        aParamW,
        aParamL
      );
  End;

{$WARN UNSAFE_TYPE ON}

//
// end of special command line stuff

Type

  TTestTracer = Class( TTracer)
  Private
    FStrings : TStrings;
  Public
    Constructor Create( aStrings: TStrings);
    Procedure   Trace( Const aMsg: String); Override;
  End;

    Constructor TTestTracer.Create( aStrings: TStrings);
    Begin
      Inherited Create;
      FStrings := aStrings;
      // If Assigned( FStrings)
      // Then FStrings.Clear;
    End;

    Procedure   TTestTracer.Trace( Const aMsg: String); // Override;
    Begin
      If Assigned( FStrings)
      Then FStrings.Add( aMsg);
    End;

//  Private

    Procedure   TFormMain.SetTestFileName( Const aValue: String);
    Begin
      If aValue <> FTestFileName
      Then Begin
        FTestFileName := aValue;
        EditTestFileName.Text := FTestFileName;
      End;
    End;

    Procedure   TFormMain.SetConnected( aValue: Boolean);
    Const
      Caps : Array[ Boolean] Of String = (
        '&Connect',
        'Dis&connect'
      );
      Connects : Array[ Boolean] Of String = (
        'Netmidi connection lost',
        'Netmidi connection made'
      );
    Begin
      If aValue <> FConnected
      Then Begin
        FConnected := aValue;
        BitBtnConnect.Caption := Caps[ FConnected];
        Log( Connects[ FConnected]);
        If FConnected And ( FCommands.Count > 0)
        Then Begin
          TimerCommand.Enabled := False;
          ProcessCommands;
        End;
      End;
    End;

    Procedure   TFormMain.SetRunning( avalue: Boolean);
    Const
      Caps : Array[ Boolean] Of String = (
        '&Start',
        '&Stop'
      );
    Begin
      If aValue <> FRunning
      Then Begin
        FRunning := aValue;
        BitBtnRunStop.Caption := Caps[ FRunning];
        TimerBeat    .Enabled := FRunning;
        If Not FRunning And ( FCommands.Count > 0)
        Then Begin
          FCommands.Clear;
          TimerCommand.Enabled := False;
        End;
        FNmProtocolDriver.ClearTransmitter;
        Log( 'Cleared all enqueued commands');
      End;
    End;

    Procedure   TFormMain.SetHostAddress( Const aValue: String);
    Begin
      If aValue <> FHostAddress
      Then Begin
        FHostAddress := aValue;
        EditHostAddress.Text := FHostAddress;
      End;
    End;

    Procedure   TFormMain.SetHostPort( Const aValue: String);
    Begin
      If aValue <> FHostPort
      Then Begin
        FHostPort := aValue;
        EditHostPort.Text := FHostPort;
      End;
    End;

    Procedure   TFormMain.SetAutoStart( aValue: Boolean);
    Begin
      If aValue <> FAutoStart
      Then Begin
        FAutoStart := aValue;
        CheckBoxAutoStart.Checked := FAutoStart;
      End;
    End;

    Procedure   TFormMain.SetAutoConnect( aValue: Boolean);
    Begin
      If aValue <> FAutoConnect
      Then Begin
        FAutoConnect := aValue;
        CheckBoxAutoConnect.Checked := FAutoConnect;
      End;
    End;

//  Private

Const

  _d = 'Defaults';

    Procedure   TFormMain.LoadIni;
    Begin
      With TIniFile.Create( IniFileName) Do
        Try
          TestFileName := ReadString( _d, 'TestFileName', TestFileName);
          HostAddress  := ReadString( _d, 'HostAddress' , HostAddress );
          HostPort     := ReadString( _d, 'HostPort'    , HostPort    );
          AutoStart    := ReadBool  ( _d, 'AutoStart'   , AutoStart   );
          AutoConnect  := ReadBool  ( _d, 'AutoConnect' , AutoConnect );
        Finally
          Free;
        End;
    End;

    Procedure   TFormMain.SaveIni;
    Begin
      With TIniFile.Create( IniFileName) Do
        Try
          WriteString( _d, 'TestFileName', TestFileName);
          WriteString( _d, 'HostAddress' , HostAddress );
          WriteString( _d, 'HostPort'    , HostPort    );
          WriteBool  ( _d, 'AutoStart'   , AutoStart   );
          WriteBool  ( _d, 'AutoConnect' , AutoConnect );
        Finally
          Free;
        End;
    End;

//  Private

    Procedure   TFormMain.Log( Const aMsg: String);
    Begin
      With MemoResults, lines Do
      Begin
        If Count > 793 // (arbitrary)
        Then Clear;
        Add( aMsg);
      End;
    End;

    Procedure   TFormMain.LogFmt( Const aFormat: String; Const anArgs: Array Of Const);
    Begin
      Log( Format( aFormat, anArgs));
    End;

    Procedure   TFormMain.BrowseForTestFile;
    Begin
      With OpenDialogTest Do
      Begin
        If TestFileName <> ''
        Then FileName := TestFileName;
        If Execute
        Then TestFileName := FileName;
      End;
    End;

    Procedure   TFormMain.DeselectSlots;
    Var
      i : Integer;
    Begin
      FSlotSelection[ 0] := True;
      For i := 1 To 3 Do
        FSlotSelection[ i] := False;
      SendSlotSelection;
    End;

    Procedure   TFormMain.SelectSlot( aSlot: Integer);
    Begin
      FSlotSelection[ aSlot] := True;
      SendSlotSelection;
      FLastSlot := aSlot;
    End;

    Procedure   TFormMain.SendSlotSelection;
    Const
      OnOffs : Array[ Boolean] Of String = (
        'Off',
        'On'
      );
    Var
      aMsg : TSlotsSelectedMessage;
      i    : Integer;
    Begin
      LogFmt(
        'sending slot selection [ %s, %s, %s, %s]',
        [
          OnOffs[ FSlotSelection[ 0]],
          OnOffs[ FSlotSelection[ 1]],
          OnOffs[ FSlotSelection[ 2]],
          OnOffs[ FSlotSelection[ 3]]
        ]
      );
      aMsg := TSlotsSelectedMessage.Create;
      With aMsg Do
      Begin
        For i := 0 To 3 Do
          IsSelected[ i] := FSlotSelection[ i];
      End;
      Try
        FNmProtocolDriver.Send( aMsg);
      Except
        On E: Exception Do
          LogFmt( 'Could not send slot selection message : %s', [ E.Message]);
      End;
    End;

    Procedure   TFormMain.SetActiveSlot( aSlot: Integer);
    Var
      aMsg : TSlotActivatedMessage;
    Begin
      LogFmt( 'Setting active slot to : %d', [ aSlot]);
      aMsg := TSlotActivatedMessage.Create;
      aMsg.ActiveSlot := aSlot;
      Try
        FNmProtocolDriver.Send( aMsg);
      Except
        On E: Exception Do
          LogFmt( 'Could not send slot activation message : %s', [ E.Message]);
      End;
    End;

    Procedure   TFormMain.SendPatch( aPatch: TPatch; aSlot: Integer);
    Var
      aPatchMessage: TPatchMessage;
    Begin
      LogFmt( 'Sending patch : %s', [ aPatch.Name]);
      aPatchMessage := TPatchMessage.Create( aPatch);
      aPatchMessage.Slot := aSlot;
      Try
        FNmProtocolDriver.Send( aPatchMessage);
      Except
        On E: Exception Do
          LogFmt( 'Could not send patch : %s', [ E.Message]);
      End;
      Inc( FLastSlot);
      If FLastSlot > 3
      Then FLastSlot := 0;
    End;

    Procedure   TFormMain.SendPatchFile( Const aPatchFileName: String; aSlot: Integer);
    Begin
      If Assigned( FPatches[ aSlot])
      Then FreeAndNil( FPatches[ aSlot]);
      FPatches[ aSlot] := TPatch.Create;
      Try
        FPatches[ aSlot].Loadpatch( aPatchFileName);
        // SelectSlot( aSlot);
        // SetActiveSlot( aSlot);
        SendPatch( FPatches[ aSlot], aSlot);
      Except
        On E: Exception Do
          LogFmt( 'Could not load patch %s : %s', [ aPatchFileName, E.Message]);
      End;
    End;

{$WARN UNSAFE_CODE OFF}

    Procedure   TFormMain.DoDriverData( aSender: TObject; aData: TByteList);
    Var
      aRawData : PBytes;
      aSize    : Integer;
      i        : Integer;
      S        : String;
    Begin
      If Connected
      Then
        With ClientSocket.Socket Do
        Begin
          aSize := aData.Size;
          GetMem( aRawData, aSize);
          Try
            S := '';
            For i := 0 To aSize - 1 Do
            Begin
              aRawData^[ i] := aData[ i];
              S := S + Format( ' %.2x', [ aData[ i]]);
            End;
            SendBuf( aRawData^, aSize);
            LogFmt( 'send:%s', [ S]);
          Finally
            FreeMem( aRawData, aSize);
          End;
        End
      Else Log( 'data not sent, no driver connected.');
    End;

    Procedure   TFormMain.DoMidiMessage( aSender: TObject; aData: TMidiMessage);
    Begin
      If Assigned( aData)
      Then LogFmt( 'Received : %s', [ aData.ClassName])
      Else Log( 'Unparsable midi message received');
    End;

    Procedure   TFormMain.DoOnUnparsable( aSender: TObject; aData: TByteList);
    Begin
      If Assigned( aData)
      Then LogFmt( 'UMD %s', [ aData.AsString])
      Else Log   ( 'Unparsable data = NIL ???');
    End;

    Procedure   TFormMain.AcceptCommandLine( aCommandLine: Atom); // Overload;
    Var
      Buf : Array[ 0 .. CommandBufSize] Of Char;
      S   : String;
    Begin
      If aCommandLine <> 0
      Then Begin
        GlobalGetAtomName( aCommandLine, Buf, CommandBufSize);
        S := StrPas( Buf);
        Try
          AcceptCommandLine( S);
        Finally
          GlobalDeleteAtom( aCommandLine);
        End;
      End;
    End;

{$WARN UNSAFE_CODE ON}

    Procedure   TFormMain.AcceptCommandLine( Const aCommandLine: String); // Overload;
    // Parse commandline assuming space characters as separators.
    // Honour quoted parts in this process.
    // Parse results will be stored into FCommands.
    Var
      i     : Integer;
      S     : String;
      State : (
        stNone,
        stInSingle,
        stInDouble,
        stSingleNest,
        stDoubleNest
      );

      Procedure SkipSpaces;
      Begin
        While ( i <= Length( aCommandLine)) And ( aCommandLine[ i] = ' ') Do
          Inc( i);
      End;

    Begin
      LogFmt( 'command line : ''%s''', [ aCommandLine]);
      With FCommands Do
      Begin
        Clear;
        FLastSlot := 0;
        i := 1;
        SkipSpaces;
        State := stNone;
        S     := '';
        While i <= Length( aCommandLine) Do
        Begin
          Case State Of

            stNone :
              Case aCommandLine[ i] Of
                ' ' : Begin
                    If Length( S) > 0
                    Then Begin
                      Add( S);
                      S := '';
                      SkipSpaces;
                      Dec( i);
                    End;
                  End;
                '''' : State := stInSingle;
                '"'  : State := stInDouble;
                Else S := S + aCommandLine[ i];
              End;

            stInSingle : Begin
                Case aCommandLine[ i] Of
                  '''' : State := stSingleNest;
                  Else S := S + aCommandLine[ i];
                End;
              End;

            stInDouble : Begin
                Case aCommandLine[ i] Of
                  '"' : State := stDoubleNest;
                  Else S := S + aCommandLine[ i];
                End;
              End;

            stSingleNest : Begin
                Case aCommandLine[ i] Of
                  '''' : Begin
                      S := S + aCommandLine[ i];
                      SkipSpaces;
                      State := stInSingle;
                    End;
                  Else Begin
                      If Length( S) > 0
                      Then Begin
                        Add( S);
                        S := '';
                      End;
                      State := stNone;
                    End;
                End;
              End;

            stDoubleNest : Begin
                Case aCommandLine[ i] Of
                  '"' : Begin
                      S := S + aCommandLine[ i];
                      SkipSpaces;
                      State := stInSingle;
                    End;
                  Else Begin
                      If Length( S) > 0
                      Then Begin
                        Add( S);
                        S := '';
                      End;
                      State := stNone;
                    End;
                End;
              End;

          End; // Case State Of
          Inc( i);
        End;
        If S <> ''
        Then Add( S);

        If Count > 0     // Strip first command, it's the application name (our namee).
        Then Delete( 0);

        For i := 0 To Count - 1 Do
          LogFmt( '  cmd : ''%s''', [ Strings[ i]]);
        Log( 'Commandline ended.');
        If Connected
        Then ProcessCommands
        Else
          With TimerCommand Do
          Begin
            Enabled := False;
            Enabled := True;
            Log( 'no NetMidi connection, commands queued for later use.');
          End;
      End;
    End;

    Procedure   TFormMain.ProcessCommands;
    Begin
      DeselectSlots;
      With FCommands Do
        While Count > 0 Do
        Begin
          ProcessCommand( Strings[ 0]);
          Delete( 0);
        End;
    End;

    Procedure   TFormMain.ProcessCommand( Const aCommand: String); // Overload;
    Var
      Cmd : String;
      Arg : String;
      i   : Integer;
    Begin
      i := Pos( '=', aCommand);
      If i > 0
      Then Begin
        Cmd := Copy( aCommand, 1, i - 1);
        Arg := Copy( aCommand, i + 1, Length( aCommand));
      End
      Else Begin
        Cmd := aCommand;
        Arg := '';
      End;
      Try
        ProcessCommand( Cmd, Arg);
      Except
        On E: Exception Do
          LogFmt( 'Error executing commnad ''%s'' : %s', [ Cmd, E.Message]);
      End;
    End;

    Procedure   TFormMain.ProcessCommand( Const aCommand, anArg: String); // Overload;
    Var
      Cmd : String;
    Begin
      If anArg <> ''
      Then LogFmt( 'processing : %s=%s', [ aCommand, anArg])
      Else LogFmt( 'processing : %s', [ aCommand]);
      Cmd := UpperCase( aCommand);
      If Cmd = 'FILE'
      Then Begin
        If FileExists( anArg)
        Then SendPatchFile( anArg, FLastSlot)
        Else LogFmt( 'Patch ''%s'' not found', [ anArg]);
      End
      Else If FileExists( ApplicationPath + 'patches\' + aCommand)
      Then SendPatchFile( ApplicationPath + 'patches\' + aCommand, FLastSlot)
      Else LogFmt( 'Unknown command, unknown patch ''%s''', [ aCommand]);
    End;


// ////////////////////////////////////////////////////////////////////////////
// Delpi area
// ////////////////////////////////////////////////////////////////////////////



{$WARN UNSAFE_CODE OFF}
{$WARN UNSAFE_TYPE OFF}
{$WARN SYMBOL_PLATFORM OFF}

procedure TFormMain.FormShow(Sender: TObject);
Var
  aCommandLine : Atom;
  aData        : Array[ 0 .. CommandBufSize] Of Char;
begin
  MemoResults.Clear;
  LoadIni;
  If AutoStart
  Then Running := True;
  If AutoConnect
  Then BitBtnConnectClick( Self);
  If ParamCount >= 1
  Then Begin
    StrPCopy( @ aData, CmdLine);
    aCommandLine := GlobalAddAtom( aData)
  End
  Else aCommandLine := 0;
  AcceptCommandLine( aCommandLine);
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  TimerBeat.Enabled := False;
  FreeAndNil( FNmProtocolDriver);
  FreeAndNil( FCommands);
  SaveIni;
  SetWindowLong( FormMain.Handle, GWL_WNDPROC, LongInt( OldWindowProc));
end;

{$WARN UNSAFE_CODE ON}
{$WARN UNSAFE_TYPE ON}
{$WARN SYMBOL_PLATFORM ON}

procedure TFormMain.BitBtnExitClick(Sender: TObject);
begin
  Running   := False;
  FStopping := True;
  ClientSocket.Close;
  Close;
end;

procedure TFormMain.BitBtnBrowseTestClick(Sender: TObject);
begin
  BrowseForTestFile;
end;

procedure TFormMain.BitBtnBeamItClick(Sender: TObject);
begin
  SendPatchFile( TestFileName, 0);
end;

{$WARN UNSAFE_CODE OFF}
{$WARN UNSAFE_TYPE OFF}

procedure TFormMain.FormCreate(Sender: TObject);
begin
  TPatchMessage.UsePdlFile( ApplicationPath + 'patch.pdl');
  CmdLineMsg := RegisterWindowMessage( SpecialSymbol);
  OldWindowProc :=
    Pointer(
      SetWindowLong( FormMain.Handle, GWL_WNDPROC, LongInt( @ NewWindowProc))
    );
  FNmProtocolDriver := TNMProtocolDriver.Create;
  FCommands := TStringList.Create;
  With FNmProtocolDriver Do
  Begin
    OnMidiMessage := DoMidiMessage;
    OnDriverData  := DoDriverData;
    OnUnparsable  := DoOnUnparsable;
  End;
  PanelTop.DoubleBuffered := True;
end;

{$WARN UNSAFE_CODE ON}
{$WARN UNSAFE_TYPE ON}

procedure TFormMain.TimerBeatTimer(Sender: TObject);
Var
  img1 : String;
  img2 : String;
begin
  img1 := ApplicationPath + 'Heart-1.bmp';
  img2 := ApplicationPath + 'Heart-2.bmp';
  Try
    FNmProtocolDriver.HeartBeat;
  Except
    On E: Exception Do
      LogFmt( 'Heart beat trouble : %s', [ E.Message]);
  End;
  FBeatState := ( FBeatState + 1) Mod 20;
  With ImageRun.Picture Do
  If FBeatState In [ 0, 1, 2, 9]
  Then Begin
    If FileExists( img1)
    Then LoadFromFile( img1);
  End
  Else Begin
    If FileExists( img2)
    Then LoadFromFile( img2);
  End;
end;

procedure TFormMain.BitBtnRunStopClick(Sender: TObject);
begin
  Running := Not Running;
end;

procedure TFormMain.ClientSocketConnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  Connected := True;
end;

procedure TFormMain.ClientSocketDisconnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  Connected := False;
end;

procedure TFormMain.ClientSocketError(Sender: TObject;
  Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
begin
  With Socket Do
    Log(
      Format(
        '** Socket error %d',
        [ ErrorCode]
      )
    );
  ErrorCode := 0;
end;

procedure TFormMain.ClientSocketLookup(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  Log( 'resolving host');
end;

{$WARN UNSAFE_CODE OFF}

procedure TFormMain.ClientSocketRead(Sender: TObject;
  Socket: TCustomWinSocket);
Var
  aData : PBytes;
  aSize : Integer;
begin
  If Not FStopping
  Then
    With Socket Do
    Begin
      aSize := ReceiveLength;
      While aSize > 0 Do
      Begin
        GetMem( aData, aSize);
        ReceiveBuf( aData^, aSize);
        Try
          FNmProtocolDriver.AcceptRawDriverData( aData, aSize);
        Finally
          FreeMem( aData, aSize);
        End;
        aSize := ReceiveLength;
      End;
  End;
end;

{$WARN UNSAFE_CODE ON}


procedure TFormMain.BitBtnConnectClick(Sender: TObject);
begin
  With ClientSocket Do
  Begin
    If Active
    Then Close
    Else Begin
      If IsDottedIp( HostAddress)
      Then Address := HostAddress
      Else Host    := HostAddress;
      Port := StrToInt( HostPort);
      Open;
    End;
  End;
end;

procedure TFormMain.EditHostAddressChange(Sender: TObject);
begin
  HostAddress := EditHostAddress.Text;
end;

procedure TFormMain.EditHostPortChange(Sender: TObject);
begin
  HostPort := EditHostPort.Text;
end;

procedure TFormMain.CheckBoxAutoStartClick(Sender: TObject);
begin
  AutoStart := CheckBoxAutoStart.Checked;
end;

procedure TFormMain.CheckBoxAutoConnectClick(Sender: TObject);
begin
  AutoConnect := CheckBoxAutoConnect.Checked;
end;

procedure TFormMain.BitBtnClsClick(Sender: TObject);
begin
  MemoResults.Clear;
end;

procedure TFormMain.TimerCommandTimer(Sender: TObject);
begin
  If Connected
  Then ProcessCommands
  Else
    With TimerCommand Do
    Begin
      Enabled := False;
      Enabled := True;
      Log( 'still no NetMidi connection, commands still queued.');
    End;
end;

End.
